""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

from CountermeasurePlugin import CountermeasurePlugin
try: import json
except ImportError: import simplejson as json
import base64

class FortiSOARTakeoverCountermeasure(CountermeasurePlugin):

     name = "FortiSOAR Takeover"
     author = "fortisoar-engg-devops@fortinet.com"
     textkey = "fortisoar.cm.takeover"
     description = "Perform a takeover operation to make this secondary node as the primary cluster node. In cluster having more than 2 nodes, you see separate incident from all the secondary nodes asking for approval for this countermeasure. However, you should approve this countermeasure only for the one node which you expect to be new cluster primary, and ignore rest of the approval requests from the remaining secondary nodes."
     sudo_requirements = ["/bin/csadm ha takeover --no-interaction"]

     def run(self):
        self.log.info("Starting FortiSOAR takeover")
        cmd = "sudo /bin/csadm ha takeover --no-interaction"
        json_path = "/opt/cyops-auth/.list_nodes.json"
        nodes = None
        try:
            with open(json_path, 'r') as json_contents:
                nodes = json.load(json_contents)
        except Exception as e:
            self.log.error(str(e))
        server_metadata = self.metadata.get("server", None)
        if server_metadata is not None:
            server_metadata = server_metadata.get("attributes", None)
        takeover_nodes = []
        if (nodes is not None) and (server_metadata is not None):
            for node in nodes:
                if node['nodeName'] in server_metadata.keys():
                    takeover_node = {}
                    takeover_node['node_name'] = node['nodeName']
                    input_choice = server_metadata[node['nodeName']]
                    # Bydefault, node will join back as passive
                    takeover_node['choice'] = 1
                    if input_choice.lower() == "active":
                        takeover_node['choice'] = 2
                    elif input_choice.lower() == "do-not-join":
                        takeover_node['choice'] = 3
                    takeover_nodes.append(takeover_node)
        json_encoded_list = json.dumps(takeover_nodes)
        b64_encoded_list = base64.b64encode(json_encoded_list)
        cmd = cmd + " --payload " + b64_encoded_list
        ret, out = self.execute(cmd)
        self.save_text_output(out)
        self.save_return_code(ret)
