#!/bin/bash
# Copyright start
# Copyright (C) 2008 - 2024 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

# After installing the "install-fortisoar.sh" script,
# if the user logs in with a different user (other than csadmin) and config-vm is not run, then display the warning message.
check_csadmin_user(){

    local s_user_name_csadmin="csadmin"
    s_user_id_root=0
    s_config_vm_script="config-vm.sh"
    s_warning_label="\033[1;33mWarning\033[0m"

    d_home_csadmin="/home/$s_user_name_csadmin"
    f_vm_configured="$d_home_csadmin/.vm_configured"
    f_vm_config_failed="$d_home_csadmin/.vm_configuration_failed"
    f_pid_vm_config="$d_home_csadmin/${s_config_vm_script}.pid"

    # Check if config vm is not successfully done, show message to complete it successfully

    if [ $EUID -eq $s_user_id_root ]; then
        # Check if config-vm.sh is one of the parents of this process
        # Do not show if parent process is config-vm itself, else message gets shown in config vm logs
        s_pstree=$(pstree -a $PPID)
        echo $s_pstree | grep --quiet $s_config_vm_script
        if [ $? -ne 0 ];then
            if [ -f "$f_vm_config_failed" ];then
                # Config VM failed
                echo -e "$s_warning_label: A failure has been detected in this instance during FortiSOAR VM configuration."
                echo "         It is recommended to resolve the issues and complete the VM configuration process before running any operation."
            elif [ -f "$f_pid_vm_config" ]; then
                # Config VM in progress
                echo -e "$s_warning_label: FortiSOAR VM configuration is in progress."
                echo "         Please allow the VM configuration process to complete before running any operation."
            elif [ ! -f "$f_vm_configured" ]; then
                # Config VM not done
                echo -e "$s_warning_label: FortiSOAR VM configuration is still pending."
                echo "         Please run the VM configuration process before running any operation."
            fi
        fi
    fi
}
