#!/bin/sh
# Copyright start
# Copyright (C) 2008 - 2024 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

show_page_first(){
    # Block the Escape key by allowing ret_code=0 only
    ret_code=2
    while [ $ret_code != 0 ];do
        $dialog --title      "$s_title_paged" \
                --ok-label   "$s_continue_label" \
                --msgbox     "$s_eula" \
                $i_dialog_height \
                $i_dialog_width
        ret_code=$?
    done

    # User pressed the "Continue" key so go to next page
    i_page_index=`expr ${i_page_index} + 1`
    continue
}

show_page_in_between(){
    # in-between pages
    $dialog --title      "$s_title_paged" \
           --yes-label  "$s_back_label" \
           --no-label   "$s_continue_label" \
           --yesno      "$s_eula" \
           $i_dialog_height \
           $i_dialog_width
    ret_code=$?
    # "Back" button returns 0
    # "Continue" button returns 1
    if [ $ret_code -eq 0 ];then
        i_page_index=`expr $i_page_index - 1`
        continue
    elif [ $ret_code -eq 1 ];then
        i_page_index=`expr $i_page_index + 1`
        continue
    fi
}

show_page_last(){
    # Last page
    if [ ${i_total_pages} -gt 1 ];then
        $dialog --title          "$s_title_paged" \
               --extra-button \
               --extra-label    "$s_accept_label" \
               --ok-label       "$s_back_label" \
               --cancel-label   "$s_do_not_accept_label" \
               --yesno          "$s_eula" \
               $i_dialog_height \
               $i_dialog_width
    else
        export DIALOG_OK=3
        $dialog --title     "$s_title_paged" \
               --yes-label  "$s_accept_label" \
               --no-label   "$s_do_not_accept_label" \
               --yesno      "$s_eula" \
               $i_dialog_height \
               $i_dialog_width
    fi
    ret_code=$?
    # "Back" button returns 0
    # "Accept" button returns 3
    # "Do not accept" button returns 1
    if [ $ret_code -eq 0 ];then
        # User pressed "Back" button.
        # Show previous page
        i_page_index=`expr $i_page_index - 1`
    elif [ $ret_code -eq 3 ];then
        # User pressed "Accept" button
        # Exit
        touch $f_eula_agreed
        clear
    elif [ $ret_code -eq 1 ];then
        # User pressed "Do not accept" button
        # Show confirmation dialog
        clear
        s_msg="$s_eula_decline_confirmation"
        $dialog --title      "| $s_title |" \
               --yes-label  "Yes" \
               --no-label   "No" \
               --yesno      "$s_msg" \
               $i_dialog_height \
               $i_dialog_width
        ret_code=$?
        if [ 0 = $ret_code ];then
            sudo halt
        fi
    fi
    return $ret_code
}

process_eula(){
    # trap Control+C
    trap "" SIGINT

    # Start with page 1
    i_page_index=1

    while true ;do
        # Read eula using index variable
        s_eula=`cat ${d_eula}/${s_eula_filename}-${i_page_index}.txt`

        # Set title according to index variable
        s_title_paged="| $s_title - Page ${i_page_index} of ${i_total_pages} |"

        case ${i_page_index} in
            ${i_total_pages})
                show_page_last
                ret_code=$?
                if [ $ret_code -eq 0 -o $ret_code -eq 1 ];then
                    continue
                else
                    break
                fi
            ;;

            1)
                show_page_first
                continue
            ;;

            *)
                show_page_in_between
                continue
            ;;
        esac
    done
}

init(){
    d_home=/home/csadmin/

    f_eula_agreed=$d_home/.eula_agreed
    d_eula=/opt/cyops/scripts/eula/

    s_title="EULA"

    s_product_name="FortiSOAR"

    s_accept_label="Accept"
    s_do_not_accept_label="Do not accept"

    s_back_label="Back"
    s_continue_label="Continue"

    i_dialog_height=35
    i_dialog_width=90

    s_eula_decline_confirmation="  You need to accept the EULA in order to use $s_product_name.\n  System will be halted if EULA is not accepted.\n  Are you sure?"

    export NCURSES_NO_UTF8_ACS=1

    s_eula_filename="eula-ascii-page"

    # Store the count of total eula pages
    i_total_pages=`ls ${d_eula}/${s_eula_filename}-*.txt | wc -l`

    dialog=dialog
}

#######
# MAIN
#######
init

if [ ! -f "$f_eula_agreed" ];then
    process_eula
fi
