#!/bin/sh
# Copyright start
# Copyright (C) 2008 - 2024 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end


init() {
    #
    # Below file is created as last step of
    # /opt/cyops/scripts/cloud/openstack/initialize_forticloud_config.py
    #
    FLAGS_DIR="/opt/cyops/configs/.flags"
    FILE_FSR_INIT_CONFIG_COMPLETED="$FLAGS_DIR/.forticloud-init-config-completed"
    FILE_CONFIG_VM_COMPLETED="$FLAGS_DIR/.vm_configured"
    FILE_FSR_INIT_CONFIG_RESTRICT_RERUN="$FLAGS_DIR/.restrict-rerun-for-forticloud-init-config"
    FILE_FIRST_BOOT_AFTER_LEAPP_UPGRADE="$FLAGS_DIR/.first-boot-after-leapp-upgrade"
}

is_first_boot() {
    if [ -f "$FILE_FSR_INIT_CONFIG_COMPLETED" ] || [ -f "$FILE_CONFIG_VM_COMPLETED" ]; then
        # 1 means False
        return 1
    fi
    # 0 means True
    return 0
}

handle_first_boot() {
    echo "Starting config VM"
    /opt/cyops/scripts/config-vm.sh --non-interactive
    echo "Starting enable embedded SME"
    csadm secure-message-exchange enable --no-interaction
    # Remove secure message exchange management API port in firewall
    csadm secure-message-exchange management-extension block-port
}

assign_static_ip_address(){
    # This function assigns static IP address for forticloud VM
    # The IP address assigned during DHCP to forticloud VM is used for static IP configuration

    # Disable the "exit on error" behavior
    set +e

    # Fetch the ip data assigned to VM via dhcp
    local s_conn_name="$(nmcli --get-values NAME conn show --active | head -1)"
    local s_ip_addr_subnet=$(nmcli --get-values IP4.ADDRESS conn show "$s_conn_name")
    local s_gateway=$(nmcli --get-values IP4.GATEWAY conn show "$s_conn_name")
    local s_current_ipv4_method=$(nmcli --get-values ipv4.method con show "$s_conn_name")

    if [ $s_current_ipv4_method == "manual" ]; then
        # IP Address is already assigned as static
        return
    fi

    echo "Assigning static IP"
    # Setting up the static IP using nmcli commmand.
    sudo nmcli connection modify "$s_conn_name" \
         ipv4.addresses $s_ip_addr_subnet \
         ipv4.gateway $s_gateway \
         ipv4.method manual

    if [ $? -ne 0 ];then 
        echo "WARNING: Failed to assign static IP to the system"
    else 
        # NetworkManager restart is required after changing network configuration.
        echo "Restarting NetworkManager service"
        systemctl restart NetworkManager
        echo "Done"
    fi

    # Enable the "exit on error" behavior
    set -e
}

add_route_to_cloud_metadata_service(){

    local s_conn_name="$(nmcli --get-values NAME conn show --active | head -1)"
    local s_device_name=$(nmcli --get-values GENERAL.DEVICES conn show "$s_conn_name")

    local ip_forticloud_metadata_service="169.254.169.254"
    local i_ip_range="31"
    local s_default_route="0.0.0.0"

    #Disable the "exit on error" behavior
    set +e

    # Check if the route entry is present for Forticloud Metadata Service
    route -n | awk '{print $1}' | grep "$ip_forticloud_metadata_service" > /dev/null 
    local i_return_code=$?
    
    #Enable back the "exit on error" behavior
    set -e

    if [ $i_return_code -eq 0 ]; then 
        # route entry for Forticloud Metadata Service is already present
        echo "The route entry for the FortiCloud Metadata Service is already present."
        return
    fi 

    echo "Adding the route entry for the FortiCloud Metadata service as it is not present."
    ip route add $ip_forticloud_metadata_service/$i_ip_range \
             via $s_default_route \
             dev $s_device_name
    echo "Done"
}

#############
#Main/main/MAIN starts here
##############

init

# Exit in case of any failure
set -e

monitor_parameters=""

if is_first_boot; then
    echo "Running first boot"
    handle_first_boot
fi

monitor_parameters=""

if [ -f "$FILE_FSR_INIT_CONFIG_COMPLETED" ]; then
    # This is subsequent boot
    # This is subsequent boot. Delete the flag to run init config script again with parameter --subsequent-boot
    echo "Running subsequent boot"
    echo "Removing file [$FILE_FSR_INIT_CONFIG_RESTRICT_RERUN]"
    rm -f $FILE_FSR_INIT_CONFIG_RESTRICT_RERUN
    monitor_parameters="--subsequent-boot"
fi

# Check if this is a first boot after leapp upgrade, as leapp actor stops unexpectedly after execution of add_route_to_cloud_metadata_service
# Hence skipping the function call for first boot after leapp upgrade.
if [ ! -f "$FILE_FIRST_BOOT_AFTER_LEAPP_UPGRADE" ]; then
    assign_static_ip_address
    add_route_to_cloud_metadata_service
fi

script_monitor_forticloud_init_config="/opt/cyops/scripts/cloud/openstack/monitor-forticloud-init-config.sh $monitor_parameters"

echo "Running script [$script_monitor_forticloud_init_config]"

sh $script_monitor_forticloud_init_config &

cron_file=/etc/cron.d/monitor-forticloud-init-config

echo "Setting cron [$cron_file]"

echo "*/1 * * * * root sh $script_monitor_forticloud_init_config" > $cron_file
