#!/bin/sh
# Copyright start
# Copyright (C) 2008 - 2024 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

logme(){
    local msg=$1
    local time_stamp=$(date)
    echo "[$time_stamp] $msg" | tee --append $f_log
}

init(){
    init_config="/opt/cyops/scripts/cloud/openstack/initialize_forticloud_config.py"
    FLAGS_DIR="/opt/cyops/configs/.flags"
    FILE_FSR_INIT_CONFIG_COMPLETED="$FLAGS_DIR/.forticloud-init-config-completed"
    FILE_FSR_INIT_CONFIG_RESTRICT_RERUN="$FLAGS_DIR/.restrict-rerun-for-forticloud-init-config"
    file_pid_init_config="/tmp/.forticloud-init-config-pid"
    pid_init_config=$(cat $file_pid_init_config 2>/dev/null)
    cron_file_monitor_init_config="/etc/cron.d/monitor-forticloud-init-config"
}

is_init_config_running(){
    local cmd_line
    cmd_line=$(ps -o command --no-heading $pid_init_config)
    if echo $cmd_line | grep -q $init_config; then
        # 0 means True
        return 0
    fi
    # 1 means False
    return 1
}

start_init_config(){
    local parameters="$1"
    logme "Starting FortiCloud init config with parameters [$parameters]"
    #start in background
    $init_config $parameters &
    pid_init_config=$!
    logme "Started FortiCloud init config with parameters [$parameters] with pid as [$pid_init_config]"
    echo $pid_init_config > $file_pid_init_config
    sleep 2
}

##############
# main/Main/MAIN starts here
##############

#
# This script gets run as a cron job. By default cron job scripts are getting path as /bin and /usr/bin.
# We use rabbitmqctl going ahead in configure script which is in /sbin. If rabbitmqctl does not get
# find into $PATH, then mq regenerate cert flushing the MQ database causing users created getting vanished.
# Hence export PATH with sbin and usr/sbin.
#
export PATH=$PATH:/sbin:/usr/sbin

monitor_parameters="$@"

f_log="/var/log/cyops/fcloud/monitor-init.log"
d_log=$(dirname "$f_log")

if [ ! -d $d_log ]; then          
    mkdir -p $d_log
fi

init

# Before going further, first check if init config script is allowed to run or not.
if [ -f "$FILE_FSR_INIT_CONFIG_RESTRICT_RERUN" ]; then
    logme "FortiCloud init config completed for parameters [$monitor_parameters] as file [$FILE_FSR_INIT_CONFIG_RESTRICT_RERUN] exist. No need to monitor now."
    logme "Removing file [$cron_file_monitor_init_config]"
    rm -f $cron_file_monitor_init_config
    exit 0
fi

if [ "X$pid_init_config" != "X" ]; then
    # PID got. Check if it is of init config script
    if is_init_config_running; then
        # init config already running, exit
        logme "FortiCloud init config is already running for parameters [$monitor_parameters]"
        exit 0
    fi
fi

start_init_config "$monitor_parameters"

exit 0
