#!/bin/sh
# Copyright start
# Copyright (C) 2008 - 2024 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

my_exit() {
    i_ret_code=$1

    # Remove call from bash profile
    sed -i '/config-vm.sh$/d' $f_bash_profile > /dev/null 2>&1

    # Cleanup temp files
    rm -f /etc/sudoers.d/csadmin

    # Enable back control C
    trap 2

    if [ 0 -eq $i_ret_code ]; then
        # Cleanup
        rm -f $f_summary
        rm -f $f_config_vm_response
        rm -f $f_pid_vm_config

        if [ $flag_interactive_mode -eq 1 ]; then
            # For some reason, when dialog is run and the config-vm script exits, typed characters are not seen on the command line.
            # pkill exits that shell making the user log in again
            pkill -KILL -u csadmin
        fi
    else
        # Non zero return code
        touch $f_vm_configuration_failed
    fi

   # Do not clean up $f_summary and $f_config_vm_response upon failure
   # They might be required for debugging
   exit $i_ret_code
}

###############################
# Messagebox functions library
###############################
show_one_liner_mbox() {
    s_msg_temp="$1"
    if [ $flag_interactive_mode -eq 1 ]; then
        s_msg_temp_len=${#s_msg_temp}
        i_width_dialog_one_liner_mbox=$(expr $s_msg_temp_len + 3)
        show_dialog_mbox "$s_title" \
                         "OK" \
                         "$s_msg_temp" \
                         6 \
                         $i_width_dialog_one_liner_mbox
    else
        # Just echo the msg on console in case of non interactive mode
        echo -e "$s_msg_temp"
    fi
}

show_dialog_passwdbox() {
    s_title_dialog_passwdbox=$1
    s_ok_label_dialog_passwdbox=$2
    s_cancel_label_dialog_passwdbox=$3
    s_msg_dialog_passwdbox=$4
    i_height_dialog_passwdbox=$5
    i_width_dialog_passwdbox=$6
    f_file=$7
    s_input_string=$8
    $f_dialog_cmd --backtitle "$s_backtitle" \
           --title "| $s_title_dialog_passwdbox |" \
           --insecure \
           --ok-label "$s_ok_label_dialog_passwdbox" \
           --cancel-label "$s_cancel_label_dialog_passwdbox" \
           --passwordbox "$s_msg_dialog_passwdbox" \
           $i_height_dialog_passwdbox \
           $i_width_dialog_passwdbox \
           "$s_input_string" 2>$f_file
}

show_dialog_inputbox() {
    s_title_dialog_inputbox=$1
    s_ok_label_dialog_inputbox=$2
    s_cancel_label_dialog_inputbox=$3
    s_msg_dialog_inputbox=$4
    i_height_dialog_inputbox=$5
    i_width_dialog_inputbox=$6
    f_file=$7
    s_input_string=$8
    $f_dialog_cmd --backtitle "$s_backtitle" \
           --title "| $s_title_dialog_inputbox |" \
           --ok-label "$s_ok_label_dialog_inputbox" \
           --cancel-label "$s_cancel_label_dialog_inputbox" \
           --inputbox "$s_msg_dialog_inputbox" \
           $i_height_dialog_inputbox \
           $i_width_dialog_inputbox \
           "$s_input_string" 2>$f_file
}

show_dialog_yesno() {
    s_title_dialog_yesno=$1
    s_yes_label_dialog_yesno=$2
    s_no_label_dialog_yesno=$3
    s_msg_dialog_yesno=$4
    i_height_dialog_yesno=$5
    i_width_dialog_yesno=$6
    $f_dialog_cmd --backtitle "$s_backtitle" \
           --title "| $s_title_dialog_yesno |" \
           --yes-label "$s_yes_label_dialog_yesno" \
           --no-label "$s_no_label_dialog_yesno" \
           --yesno "$s_msg_dialog_yesno" \
           $i_height_dialog_yesno \
           $i_width_dialog_yesno
    return $?
}

show_dialog_mbox() {
    if [ $flag_interactive_mode -eq 1 ]; then
        s_title_dialog_mbox=$1
        s_ok_label_dialog_mbox=$2
        s_msg_dialog_mbox=$3
        i_height_dialog_mbox=$4
        i_width_dialog_mbox=$5
        $f_dialog_cmd --colors --backtitle "$s_backtitle" \
               --title "| $s_title_dialog_mbox |" \
               --ok-label "$s_ok_label_dialog_mbox" \
               --msgbox "$s_msg_dialog_mbox" \
               $i_height_dialog_mbox \
               $i_width_dialog_mbox
    else
        echo "$3\n"
    fi

}

show_dialog_progress() {
    i_perc=$1
    s_title_dialog_progress=$2
    s_msg_dialog_progress=$3
    echo $i_perc | $f_dialog_cmd --backtitle "$s_backtitle" \
                        --title "| $s_title_dialog_progress |" \
                        --gauge "$s_msg_dialog_progress" \
                        $i_height_dialog \
                        $i_width_dialog \
                        0
}

show_dialog_infobox() {
    s_title_dialog_infobox=$1
    s_msg_dialog_infobox=$2
    i_height_dialog_infobox=$3
    i_width_dialog_infobox=$4
    $f_dialog_cmd --backtitle "$s_backtitle" \
           --title "| $s_title_dialog_infobox |" \
           --infobox "$s_msg_dialog_infobox" \
           $i_height_dialog_infobox \
           $i_width_dialog_infobox
}

# Form to take five inputs
show_dialog_form_get_five_inputs() {
    # Multiple input boxes in single form and set the size of form
    # and inputboxes.
    # This function needs below listed variables set before calling
    # s_dialog_form_title="Set static IP"
    # s_dialog_form_text="\nPlease enter static IP details"
    # s_dialog_form_label_input1="Static IP* :"
    # s_dialog_form_label_input2="Gateway*   :"
    # s_dialog_form_label_input3="Netmask*   :"
    # s_dialog_form_label_input4="DNS1*      :"
    # s_dialog_form_label_input5="DNS2       :"
    # f_dialog_form_output="output.txt"
    # i_dialog_form_edit_start=14
    # i_dialog_form_edit_end=21
    rm -f $f_dialog_form_output
    $f_dialog_cmd   --title "| $s_dialog_form_title |" \
                    --backtitle "$s_backtitle" \
                    --ok-label "Next" \
                    --no-cancel \
                    --stdout \
                    --form "$s_dialog_form_text" \
                    $i_height_dialog \
                    $i_width_dialog \
                    5 \
                    "$s_dialog_form_label_input1 " 1 1 "$s_dialog_form_value_input1" 1 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    "$s_dialog_form_label_input2 " 2 1 "$s_dialog_form_value_input2" 2 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    "$s_dialog_form_label_input3 " 3 1 "$s_dialog_form_value_input3" 3 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    "$s_dialog_form_label_input4 " 4 1 "$s_dialog_form_value_input4" 4 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    "$s_dialog_form_label_input5 " 5 1 "$s_dialog_form_value_input5" 5 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    > $f_dialog_form_output
    return $?
}

show_dialog_form() {
    # Multiple input boxes in single form and set the size of form
    # and inputboxes.
    i_height_dialog_form=$i_height_dialog
    i_width_dialog_form=$i_width_dialog
    i_height_inputbox_form=$i_height_inputbox
    i_width_inputbox_form=$i_width_inputbox
    s_title_dialog_form=$1
    s_form_string_dialog_form=$2
    s_inputbox1_dialog_form=$3
    s_inputbox2_dialog_form=$4
    f_tmp_output=$5
    $f_dialog_cmd  --separate-widget $'\n' --title "| $s_title_dialog_form |" \
         --backtitle "$s_backtitle" --ok-label "Ok" --cancel-label "Skip" \
         --stdout --form "$s_form_string_dialog_form" $i_height_dialog_form $i_width_dialog_form 14 \
         "$s_inputbox1_dialog_form " 1 1 "$s_input1_dialog_form" 1 $i_height_inputbox_form $i_width_inputbox_form 0 \
         "$s_inputbox2_dialog_form " 2 1 "$s_input2_dialog_form" 2 $i_height_inputbox_form $i_width_inputbox_form 0 \
         ""                          3 1 ""                      3 0                       0                      0 \
         "$s_subtitle_dialog_form "  4 1 ""                      4 0                       0                      0 \
         "$s_help1_dialog_form"      5 1 ""                      5 0                       0                      0 \
         "$s_help2_dialog_form"      6 1 ""                      6 0                       0                      0 \
         ""                          7 1 ""                      7 0                       0                      0 \
         "$s_help3_dialog_form"      8 1 ""                      8 0                       0                      0 \
         "$s_help4_dialog_form"      9 1 ""                      9 0                       0                      0 \
         "$s_help5_dialog_form"     10 1 ""                     10 0                       0                      0 \
         "$s_help6_dialog_form"     11 1 ""                     11 0                       0                      0 \
         "$s_help7_dialog_form"     12 1 ""                     12 0                       0                      0 \
         "$s_help8_dialog_form"     13 1 ""                     13 0                       0                      0 \
         > $f_tmp_output
}

# Form to take two passwords
show_dialog_form_get_two_inputs() {
    rm -f $f_dialog_form_output
    $f_dialog_cmd   --title "| $s_title |" \
                    --backtitle "$s_backtitle" \
                    $s_cancel_button_option \
                    --stdout \
                    $s_form_type "$s_dialog_form_text" \
                    $i_height_dialog \
                    $i_width_dialog \
                    2 \
                    "$s_dialog_form_label_input1 " 1 1 "$s_dialog_form_value_input1" 1 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    "$s_dialog_form_label_input2 " 2 1 "$s_dialog_form_value_input2" 2 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    > $f_dialog_form_output
    return $?
}
# Messagebox functions library END
###############################

is_secure_message_exchange_instance() {
    flag_is_secure_message_exchange="false"
    for name in $all_names_for_current_flavor
    do
        if [ $name = "secure-message-exchange" ]; then
            # 0 means true
            flag_is_secure_message_exchange="true"
            break
        fi
    done
}

#
#  This sub-routine will return "true" for enterprise as well as securonix
#
is_enterprise_instance() {
    flag_is_enterprise="false"
    for name in $all_names_for_current_flavor
    do
        if [ $name = "enterprise" ]; then
            # 0 means true
            flag_is_enterprise="true"
            break
        fi
    done
}

remove_from_summary() {
    s_label="$1"
    sed -i "/^$s_label -/d" $f_summary
}

add_to_summary() {
    s_label="$1"
    s_value="$2"
    i_length_label=${#s_label}

    # X co-ordinate on screen at which to display the value
    i_value_display_offset_x_axis=30
    i_count_filler=`expr $i_value_display_offset_x_axis - $i_length_label - 2`
    s_offset=$(printf -- '-%.0s' $(seq 1 $i_count_filler))
    echo "${s_label} ${s_offset} : $s_value" >> $f_summary
    i_count_summary_lines=`expr $i_count_summary_lines + 1`
}

add_to_execution_queue() {
    # Please ensure that value of s_execution_anchor is UNIQUE
    s_execution_anchor="$1"
    s_msg="$2"
    s_cmd="$3"

    # The response file will have tagged entries like:
    # HOSTNAME-LABEL:Change hostname
    # HOSTNAME-CMD:timeout 900 csadm hostname --set def_hostname
    # this is required for the config script to detect and overwrite default inputs in case custom inputs are given.
    # Ex: if custom input conf has
    # hostname=cust_hostname
    # config script will replace lines marked by HOSTNAME-LABEL and HOSTNAME-CMD using inputs provided by custom conf
    # Look for pre-existing anchor, if found, need perform replacement else append
    grep --quiet "^${s_execution_anchor}-LABEL:" $f_config_vm_response
    if [ $? -eq 0 ]; then
        # Found the anchor, so replace existing label and command
        sed -i "s/${s_execution_anchor}-LABEL:.*/${s_execution_anchor}-LABEL:$s_msg/g" $f_config_vm_response
        sed -i "s/${s_execution_anchor}-CMD:.*/${s_execution_anchor}-CMD:$s_cmd/g" $f_config_vm_response
    else
        # Response file will have display message as first line and command as second
        echo "${s_execution_anchor}-LABEL:${s_msg}" >> $f_config_vm_response
        echo "${s_execution_anchor}-CMD:${s_cmd}" >> $f_config_vm_response
    fi
}

# This is to address security vulnerability: Same ssh keys are present on all FSR appliances
# Need to remove them and restart sshd which regenerates all ssh keys so that they are unique per appliance
refresh_ssh_keys() {
    a_ssh_key_algorithms="ecdsa \
                          ed25519 \
                          rsa"

    for s_ssh_key_algorithm in $a_ssh_key_algorithms; do
        f_key="/etc/ssh/ssh_host_${s_ssh_key_algorithm}_key"
        rm -f "$f_key"
        rm -f "$f_key.pub"

        # Keys are not auto generated in some hypervisors
        # so need to generate it using ssh-keygen
        ssh-keygen -t $s_ssh_key_algorithm -N '' -f "$f_key"
    done

    systemctl restart sshd
}

refresh_uuids(){
    # change csadmin uuid
    default_uuid="ce5a842b-fbbc-4c6e-8d35-a0438fcdf80c"
    new_uuid=`uuidgen`
    . /opt/cyops/adm/transaction.sh

    # In a rare scenario, a token entry was found for default uuid. Since token table has a foreign key constraint on
    # userinfo table
    PGPASSWORD=${s_pg_pwd} psql -U cyberpgsql -d das -c "delete from token where uuid='$default_uuid'"
    PGPASSWORD=${s_pg_pwd} psql -U cyberpgsql -d das -c "update users set uuid='$new_uuid' where uuid='$default_uuid'; \
        update userstatus set uuid='$new_uuid' where uuid='$default_uuid'; \
        update userinfo set uuid='$new_uuid' where uuid='$default_uuid'; \
        update configprops set value_value='$new_uuid' where value_value='$default_uuid'; \
        update nfauser set uuid='$new_uuid' where uuid='$default_uuid'; \
        update nfa_status set uuid='$new_uuid' where uuid='$default_uuid';"
    PGPASSWORD=${s_pg_pwd} psql -U cyberpgsql -d venom -c "update actors set user_id='$new_uuid' where user_id='$default_uuid';"
}

update_invalid_proxy_list() {
    i_count_invalid_proxy=`expr $i_count_invalid_proxy + 1`
    l_invalid_proxy="$l_invalid_proxy\n$s_label_proxy"
}

validate_values_set_proxy() {
    s_protocol=$1
    s_label_proxy=$2
    s_line=$3

    if [ $s_protocol = "noproxy" ]; then
        # match the regex
        if [[ ! "$s_line" =~ $regex_noproxy ]]; then
            # regex does not match
            flag_invalid_input=1
            update_invalid_proxy_list
            return 1
        fi
    else
        if [ ! -z $s_line ]; then
            # match the regex
            if [[ ! "$s_line" =~ $regex_url ]]; then
                # regex does not match
                flag_invalid_input=1
                update_invalid_proxy_list
                return 1
            fi
        fi # if [ ! -z $s_line ]; then
    fi # if [ $s_protocol = "noproxy" ]; then
}

validate_input_set_proxy() {
    flag_invalid_input=0
    i_count_invalid_proxy=0
    l_invalid_proxy=""

    validate_values_set_proxy http "$s_label_proxy" $s_proxy_http

    validate_values_set_proxy noproxy "$s_label_no_proxy" $s_proxy_noproxy

    if [ $i_count_invalid_proxy -gt 0 ]; then
        s_message_invalid_proxy="\nInvalid data in:$l_invalid_proxy\n"
        return 1
    fi
}

extract_values_set_proxy() {
    s_protocol=$1
    s_line=$2

    if [ $s_protocol = "noproxy" ]; then
        # match the regex
        if [[ "$s_line" =~ $regex_noproxy ]]; then
            # regex match
            i_counter_proxy=`expr $i_counter_proxy + 1`
            s_msg="Set No Proxy"
            s_cmd="csadm network set-no-proxy --host $s_line"
            add_to_summary "No Proxy" "$s_line"
            add_to_execution_queue "NOPROXY" "$s_msg" "$s_cmd"
        fi
    else
        if [ ! -z $s_line ]; then
            # match the regex
            if [[ "$s_line" =~ $regex_url ]]; then
                # regex match
                echo $s_line | grep -q @
                if [ $? -eq 0 ]; then
                    # URL with username password found
                    s_data=`echo $s_line | sed "s/^\(.*\):\/\/\(.*\):\(.*\)@\(.*\):\(.*\)/\1 \2 \3 \4 \5/g"`
                    s_protocol_actual=`echo $s_data | cut -d' ' -f1`
                    s_username=`echo $s_data | cut -d' ' -f2`
                    s_password=`echo $s_data | cut -d' ' -f3`
                    s_host=`echo $s_data | cut -d' ' -f4`
                    i_port=`echo $s_data | cut -d' ' -f5`
                    s_creds="--user $s_username --password $s_password"

                    # To mask the password in the summary
                    s_line="$s_protocol_actual://$s_username:*****@$s_host:$i_port"
                else
                    # URL without username password found
                    s_data=`echo $s_line | sed "s/^\(.*\):\/\/\(.*\):\(.*\)/\1 \2 \3/g"`
                    s_protocol_actual=`echo $s_data | cut -d' ' -f1`
                    s_host=`echo $s_data | cut -d' ' -f2`
                    i_port=`echo $s_data | cut -d' ' -f3`
                fi
            fi

            i_counter_proxy=`expr $i_counter_proxy + 1`
            add_to_summary "${s_protocol^^} Proxy" "$s_line"
            s_msg="Set ${s_protocol^^} Proxy"
            s_cmd="csadm network set-${s_protocol}-proxy --protocol ${s_protocol_actual} --host $s_host --port $i_port $s_creds"
            add_to_execution_queue "${s_protocol^^}PROXY" "$s_msg" "$s_cmd"
        fi # if [ ! -z $s_line ]; then
    fi # if [ $s_protocol = "noproxy" ]; then
}

set_proxy() {
    if [ $flag_interactive_mode -eq 1 ]; then

        s_proxy_noproxy=""
        s_proxy_https=""
        s_proxy_http=""
        s_proxy=""

        #check if proxy is already set in /etc/environment
        s_proxy=$(grep ^http_proxy ${f_etc_environment})
        if [ $? -eq 0 ]; then
            # ouput of grep is - http_proxy='http://pxuser:root123@10.132.255.200:808'
            s_proxy=$(echo $s_proxy | cut -d "=" -f 2 | sed -e "s/'//g" -e "s/\"//g")
            # After processing output - http://pxuser:root123@10.132.255.200:808
            s_proxy_http=$s_proxy
        fi

        s_proxy=$(grep ^https_proxy ${f_etc_environment})
        if [ $? -eq 0 ]; then
            s_proxy=$(echo $s_proxy | cut -d "=" -f 2 | sed -e "s/'//g" -e "s/\"//g")
            s_proxy_https=$s_proxy
        fi

        s_proxy=$(grep ^no_proxy ${f_etc_environment})
        if [ $? -eq 0 ]; then
            s_proxy=$(echo $s_proxy | cut -d "=" -f 2 | sed -e "s/'//g" -e "s/\"//g")
            s_proxy_noproxy=$s_proxy
        fi
 
        # Interactive mode
        # Get the http and noproxy URL as inputs.
        # Set the example of the user provided inputs
        s_title_form_dialog="Configure Proxy"
        s_backtitle_form_dialog="Configure Proxy"
        s_label_proxy="Proxy"
        s_label_no_proxy="No Proxy"
        s_input1="$s_label_proxy  :"
        s_input2="$s_label_no_proxy    :"
        i_height_inputbox=15
        i_width_inputbox=60
        s_subtitle_dialog_form="Usage:"
        # s_proxy_http and s_proxy_https are same.
        s_input1_dialog_form=$s_proxy_http      
        s_input2_dialog_form="127.0.0.1,localhost"
        if [ ! -z $s_proxy_noproxy ]; then
            s_input2_dialog_form=$s_proxy_noproxy
        fi
        f_output="output.txt"
        s_help1_dialog_form="1. Proxy    http://[<userid>:<password>@](<ip>|<fqdn>):<port>"
        s_help2_dialog_form="2. No Proxy   [<ip1>/<fqdn1>][,<ip2>/<fqdn2>]"
        s_help3_dialog_form="Please note:"
        s_help4_dialog_form="1. Do not use any special character in proxy except / @ and :"
        s_help5_dialog_form="2. Do not use any special character in no proxy except comma (,) between two"
        s_help6_dialog_form="   hostnames or an ip address and a hostname."
        s_help7_dialog_form="3. 127.0.0.1,localhost will be added to no proxy list even if you choose to skip"
        s_help8_dialog_form="   the configuration"
        
        

        while true; do
            i_counter_proxy=0
            
            # Clean up the summary for proxy lines if any
            remove_from_summary "Proxy"
            remove_from_summary "No Proxy"

            show_dialog_form "$s_title_form_dialog" "$s_backtitle_form_dialog" "$s_input1" "$s_input2" "$f_output"
            if [ $? -eq 0 ]; then
                # User pressed OK
                s_proxy_noproxy=$(cat $f_output | sed -n '2p')
                s_proxy_http=$(cat $f_output | sed -n '1p')
                s_proxy_https=$(cat $f_output | sed -n '1p')
                validate_input_set_proxy
                if [ $? -ne 0 ]; then
                    i_dialog_height=`expr $i_count_invalid_proxy + 7`
                    i_dialog_width=50
                    show_dialog_mbox "$s_title" \
                                     "OK" \
                                     "$s_message_invalid_proxy" \
                                     $i_dialog_height \
                                     $i_dialog_width
                    s_input1_dialog_form=$s_proxy_http
                    s_input2_dialog_form=$s_proxy_noproxy
                    continue
                else
                    # All proxy inputs are good
                    break
                fi
            else
                # User pressed Skip
                break
            fi # if [ $i_response_dialog_form -eq 0 ]; then
            rm -f $f_output
        done
    else
        # Non Interactive mode
        # proxy_https, proxy_http and proxy_noproxy are sourced from the conf file
        if [ ! -z "$proxy" ]; then
            # Found "proxy_https=" in the conf
            s_proxy_https=$proxy
            s_label_https_proxy="proxy"
        fi

        if [ ! -z "$proxy" ]; then
            # Found "proxy_http=" in the conf
            s_proxy_http=$proxy
            s_label_http_proxy="proxy"
        fi

        if [ ! -z "$proxy_noproxy" ]; then
            # Found "proxy_noproxy=" in the conf
            s_proxy_noproxy=$proxy_noproxy
            s_label_no_proxy="proxy_noproxy"
        fi

        # Perform validation of the inputs if at least one input is mentioned
        if [ ! -z "$s_proxy_http" -o ! -z "$s_proxy_noproxy" ]; then
            validate_input_set_proxy
            if [ $? -ne 0 ]; then
                l_failed_inputs="${l_failed_inputs}${s_message_invalid_proxy}"
                flag_invalid_input=1
                return 1
            fi
        fi
    fi

    # process the input if mentioned else mark as skipped
    if [ ! -z "$s_proxy_https" ]; then
        extract_values_set_proxy https $s_proxy_https
    else
        add_to_summary "HTTPS Proxy" "[Skipped]"
    fi

    if [ ! -z "$s_proxy_http" ]; then
        extract_values_set_proxy http $s_proxy_http
    else
        add_to_summary "HTTP Proxy" "[Skipped]"
    fi

    if [ ! -z "$s_proxy_noproxy" ]; then
        extract_values_set_proxy noproxy $s_proxy_noproxy
    fi
}

is_valid_ipv6()
{
    local ipv6_addr=$1
    # following code will through exception if wrong IPv6 address. i.e return code is non-zero
    python3 -c "import ipaddress; ipaddress.IPv6Address('$ipv6_addr');" >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        # 0 = true
        echo "0"
    else
        # 1 = false
        echo "1"
    fi
}

match_hostname_ip() {
    string="$1"
    type=$2

    if [ "hostname" = "$type" ]; then
        [[ ! "$string" =~ $regex_hostname ]] && return 1 || return 0
    elif [ "both" = "$type" ]; then
        # Try to match hostname first. If that fails try matching IP.
        # Return 1 if that fails
        valid_ipv6=`is_valid_ipv6 "$string"`
        [[ "$string" =~ $regex_hostname ]] && return 0 \
        || \
        [[ "$string" =~ $regex_ipv4 || $valid_ipv6 =~ 0 ]] && return 0 \
        || return 1
    fi
}

validate_input_get_dns() {
    if [ -z "$s_dns" ]; then
        s_get_dns_error_text="DNS hostname or IP cannot be empty"
        return 1
    else
        match_hostname_ip "$s_dns" both
        if [ 0 -ne $? ]; then
            s_get_dns_error_text="Invalid DNS hostname or IP"
            return 1
        fi
    fi
}

get_dns() {
    if [ $flag_interactive_mode -eq 1 ]; then
        # Interactive mode
        while true; do
            f_dns=dns
            show_dialog_inputbox "$s_title" \
                                 "OK" \
                                 "Skip" \
                                 "\nEnter DNS server name or IP address [Optional]:\n\nNote: The DNS entries here are only for the initial configuration. \n
          After the instance is rebooted, the peer DNS identified by \n
          the Network Manager will be used." \
                                 $i_height_dialog \
                                 $i_width_dialog \
                                 $f_dns \
                                 ""
            if [ $? -eq 0 ]; then
                # User pressed "OK"
                s_dns=`cat $f_dns`
                if [ ! -z "$s_dns" ]; then
                    validate_input_get_dns "$s_dns"
                    if [ 0 -ne $? ]; then
                        # Invalid input
                        show_one_liner_mbox "\n$s_get_dns_error_text"
                        continue
                    fi
                    # Valid input
                    break
                else
                    # No input
                    break
                fi
            else
                # User pressed "skip"
                break
            fi
        done
        rm -f $f_dns
    else
        # Non interactive mode
        # dns is sourced from the conf file
        if [ ! -z "$dns" ]; then
            # dns= is present in config file
            s_dns=$dns
            validate_input_get_dns "$s_dns"
            if [ $? -ne 0 ]; then
                l_failed_inputs="$l_failed_inputs\n$s_get_dns_error_text"
                flag_invalid_input=1
                return 1
            fi
        fi
    fi
    if [ ! -z "$s_dns" ]; then
        s_msg="Update network configuration"
        s_cmd="$s_timeout_cmd csadm hostname --dns-name $s_dns"
        add_to_execution_queue "UNC" "$s_msg" "$s_cmd"
        add_to_summary "DNS server" "$s_dns"
    else
        # If s_dns is empty
        add_to_summary "DNS server" "[Skipped]"
    fi
}

validate_input_get_hostname() {
    if [ -z "$s_new_hostname" ]; then
        s_get_hostname_error_text="Hostname cannot be empty"
        return 1
    else
        match_hostname_ip "$s_new_hostname" hostname
        if [ 0 -ne $? ]; then
            s_get_hostname_error_text="Invalid hostname"
            return 1
        fi
    fi
}

insert_fqhn_into_db() {
    # psql is available only on enterprise
    if [ "$flag_is_enterprise" = "true" ]; then
        . /opt/cyops/adm/transaction.sh
        # If instance hostname already exist then it continue the code else make the new hostname entry in to the dynamic_variable_dynamicvariable table.
        PGPASSWORD=${s_pg_pwd} psql -U cyberpgsql sealab -t -A -c 'SELECT value FROM dynamic_variable_dynamicvariable WHERE name = '\''Server_fqhn'\'';' | grep -w $s_hostname
        if [ $? -eq 0 ]; then
            continue
        fi
        # Make new entry of hostname into the dynamic_variable_dynamicvariable table.
        PGPASSWORD=${s_pg_pwd} psql -U cyberpgsql sealab -c "insert into dynamic_variable_dynamicvariable (name, value) values ('Server_fqhn', '$s_hostname') ON CONFLICT (name) DO UPDATE SET value = '$s_hostname' ;"

    fi
}

set_up_mq_for_workflow() {

    if [ "X$flag_is_secure_message_exchange" = "Xtrue" ]; then
        # No need to do anything for SME.
        return
    fi

    #
    # In 6.6.0 release, we decided to use the MQ for workflow instead of the redis for queuing.
    # In order to achieve this, we need to enable the MQ to listen for TCP+TLS traffic on some port
    # so that secondary node will get the queuing details. For this, we need to do the changes for
    # MQ to enable to listen for TCP+TLS traffic on port 5671. Please note that, we are doing this
    # changes in Config VM as we want to have certs generation as per the environment.
    # We are not opening the port 5671 in firewalld. We will open this during the HA allowlist
    # operation.
    #

    $s_timeout_cmd csadm mq certs generate

    # Above command generate the certs in below directory. Change the perms.

    chown -R root:rabbitmq /opt/cyops/configs/rabbitmq/ssl/
}

set_up_messaging_broker() {
    $s_timeout_cmd csadm mq --rabbitmq-hostname
    insert_fqhn_into_db
    set_up_mq_for_workflow
}

change_hostname() {
    export s_propogate_changes="true"
    $s_timeout_cmd csadm hostname --set $s_hostname
    #
    # The following statement update/add the hostname in the file /etc/profile file if \
    # hostname will change during the configuration of wizard.
    # enterprise + change default hostname in config vm = hostname set in /etc/profile
    #
    $s_timeout_cmd csadm mq --rabbitmq-hostname
    insert_fqhn_into_db
    set_up_mq_for_workflow
}

get_hostname() {
    s_hostname=`hostname`
    if [ $flag_interactive_mode -eq 1 ]; then
        # interactive mode
        f_new_hostname="new_hostname"
        while true; do
            show_dialog_inputbox "$s_title" \
                                 "OK" \
                                 "Skip" \
                                 "\nEnter new hostname [Optional]:\n\nNote: Ensure that the hostname that you provide is resolvable.\n
      If the hostname gets resolved by a DNS server, ensure that \n      you provide only the node name and not the complete FQDN." \
                                 $i_height_dialog \
                                 $i_width_dialog \
                                 $f_new_hostname \
                                 "$s_hostname"
            if [ $? -eq 0 ]; then
                # User pressed "ok"
                s_new_hostname=`cat $f_new_hostname`
                validate_input_get_hostname $s_new_hostname
                if [ $? -ne 0 ]; then
                    # Invalid input
                    show_one_liner_mbox "\n$s_get_hostname_error_text."
                    continue
                fi
                # hostname value is changed from default and is valid
                break
            else
                # User pressed "skip"
                break
            fi
        done
        rm -f $f_new_hostname
    else
        # Non interactive mode
        if [ ! -z "$hostname" ]; then
            # hostname= is present in config file
            s_new_hostname=$hostname
            validate_input_get_hostname $s_new_hostname
            if [ $? -ne 0 ]; then
                l_failed_inputs="$l_failed_inputs\n$s_get_hostname_error_text"
                flag_invalid_input=1
                return 1
            fi
        fi
    fi

    if [ -z "$s_new_hostname" -o "$s_new_hostname" = "$s_hostname" ]; then
        # User either pressed "skip" in interactive mode or
        # conf file does not have hostname field in non-interactive mode or
        # previous and new hostnames are same
        add_to_summary "Hostname" "[No Change]"
        s_msg="Refresh MQ nodename"
        s_cmd="set_up_messaging_broker"
        add_to_execution_queue "HOSTNAME" "$s_msg" "$s_cmd"
    elif [ "$s_new_hostname" != "$s_hostname" ]; then
        s_hostname="$s_new_hostname"
        add_to_summary "Hostname" "$s_hostname"
        s_msg="Change hostname and refresh MQ nodename"
        s_cmd="change_hostname"
        add_to_execution_queue "HOSTNAME" "$s_msg" "$s_cmd"
    fi
}

validate_input_get_message_queue_credentials() {
    s_type="$1"

    if [ "$s_type" = "userid" ]; then
        if [ -z "$s_message_queue_user_id" ]; then
            s_message_queue_error_text="Message queue user id cannot be empty"
            return 1
        fi
    elif [ "$s_type" = "password" ]; then
        if [ -z "$s_message_queue_password" ]; then
            s_message_queue_error_text="Message queue password cannot be empty"
            return 1
        elif [ ${#s_message_queue_password} -lt $i_password_length_mininum ] ; then
            # Password minimum length check
            s_message_queue_error_text="Message queue password must be of at least $i_password_length_mininum characters"
            return 1
        fi

        if [ $flag_interactive_mode -eq 1 ]; then
            if [ -z "$s_message_queue_password_confirm" -o \
                 "$s_message_queue_password" != "$s_message_queue_password_confirm" ]; then
                s_message_queue_error_text="Message queue password doesn't match. Please enter password again"
                return 1
            fi
        fi
    fi
}

generate_messaging_credentials() {
    $s_timeout_cmd $s_cmd_message_queue_user_add --name $s_message_queue_user_id --password $s_message_queue_password
    i_ret_code=$?
    if [ $i_ret_code -ne 0 ]; then
        return $i_ret_code
    fi
    # Hide rabbitmq user password in log file.
    local s_message_queue_user_password=$(grep "$s_cmd_message_queue_user_add" $f_log)
    if [ -z "$s_message_queue_user_password" ]; then
        return
    fi
    # Get the last field of the string.
    local s_tmp_message_queue_user_password=$(echo $s_message_queue_user_password | awk '{print $NF}')
    # Replace the password with **** in s_tmp_message_queue_user_password variable.
    local s_tmp_message_queue_cmd=$(echo $s_message_queue_user_password | sed "s/$s_tmp_message_queue_user_password/'****'/g")
    # Replace the complete the string with new string s_tmp_message_queue_cmd in variable.
    sed -i "s/$s_message_queue_user_password/'$s_tmp_message_queue_cmd'/g" $f_log
}

get_message_queue_credentials() {
    if [ $flag_interactive_mode -eq 1 ]; then
        # Interactive mode
        s_message_queue_user_id="admin"
        f_message_queue_user_id="message_queue_user_id"
        while true; do
            show_dialog_inputbox "$s_title" \
                                 "OK" \
                                 "Skip" \
                                 "\nEnter Username for messaging configuration [Optional]:" \
                                 $i_height_dialog \
                                 $i_width_dialog \
                                 $f_message_queue_user_id \
                                 "$s_message_queue_user_id"
            if [ $? -eq 0 ]; then
                s_message_queue_user_id=`cat $f_message_queue_user_id`
                rm -f $f_message_queue_user_id
                validate_input_get_message_queue_credentials "userid"
                if [ $? -eq 0 ]; then
                    # Valid input
                    add_to_summary "Username" "$s_message_queue_user_id"
                    break
                else
                    # Invalid input
                    show_one_liner_mbox "\n$s_message_queue_error_text"
                    continue
                fi
            else
                # User pressed "skip" set username as admin 
                add_to_summary "Username" "admin"
                break
             fi
        done

        # Once user enters the userid and moves to password, there is no provision to backout
        # this needs to be addressed
        s_dialog_form_text="\nPlease enter the password:\n(Fields marked with * are mandatory)"
        s_dialog_form_label_input1="Enter password*   :"
        s_dialog_form_value_input1=""
        s_dialog_form_label_input2="Confirm password* :"
        s_dialog_form_value_input2=""
        s_cancel_button_option="--no-cancel"
        s_form_type="--insecure --passwordform"
        f_dialog_form_output="output.txt"
        i_dialog_form_edit_start=21
        i_dialog_form_edit_end=63
        while true; do
            show_dialog_form_get_two_inputs
            s_message_queue_password=$(cat $f_dialog_form_output | sed -n '1p' )
            s_message_queue_password_confirm=$(cat $f_dialog_form_output | sed -n '2p' )
            rm -f $f_dialog_form_output
            validate_input_get_message_queue_credentials "password"
            if [ $? -ne 0 ]; then
                # Invalid input
                show_one_liner_mbox "\n$s_message_queue_error_text"
                continue
            else
                # Valid input
                add_to_summary "Password" "********"
                break
            fi
        done
    else
        # Non interactive mode
        # message_queue_user_id and message_queue_password are sourced from the conf file
        l_failed_inputs_mq_creds=""
        s_message_queue_user_id=$message_queue_user_id
        validate_input_get_message_queue_credentials "userid"
        if [ $? -ne 0 ]; then
            # Invalid input
            l_failed_inputs_mq_creds="$l_failed_inputs_mq_creds\n$s_message_queue_error_text"
            flag_invalid_input=1
        fi

        s_message_queue_password=$message_queue_password
        validate_input_get_message_queue_credentials "password"
        if [ $? -ne 0 ]; then
            # Invalid input
            l_failed_inputs_mq_creds="$l_failed_inputs_mq_creds\n$s_message_queue_error_text"
            flag_invalid_input=1
        fi

        if [ $flag_invalid_input -eq 1 ]; then
            l_failed_inputs="${l_failed_inputs}${l_failed_inputs_mq_creds}"
            return 1
        fi
    fi

    # Run in case of valid inputs
    if [ ! -z "$s_message_queue_user_id" ]; then
        s_msg="Generate messaging credentials"
        s_cmd="generate_messaging_credentials"
        add_to_execution_queue "GMC" "$s_msg" "$s_cmd"
    fi
}

validate_input_get_message_queue_port() {
    s_message_queue_port_type="$1"
    i_message_queue_port="$2"

    # Check if the variable is empty
    if [ -z "$i_message_queue_port" ]; then
        l_failed_inputs_message_queue_ports="$l_failed_inputs_message_queue_ports\n$s_message_queue_port_type cannot be empty. Setting it back to the default"
        return 1
    fi

    # Check if the variable is non integer
    if [ ! -z "${i_message_queue_port//[0-9]}" ]; then
        l_failed_inputs_message_queue_ports="$l_failed_inputs_message_queue_ports\n$s_message_queue_port_type should be an integer"
        return 1
    fi

    # Check if port number does not exceed maximum allowed port number
    if [ $i_message_queue_port -gt $i_message_queue_port_max ]; then
        l_failed_inputs_message_queue_ports="$l_failed_inputs_message_queue_ports\n$s_message_queue_port_type cannot be greater than $i_message_queue_port_max"
        return 1
    else
        # Check if the port is already in use
        lsof -i :$i_message_queue_port >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            l_failed_inputs_message_queue_ports="$l_failed_inputs_message_queue_ports\n$s_message_queue_port_type is already in use"
            return 1
        fi
        return 0
    fi
}

validate_input_get_message_queue_ports() {
    i_count_invalid_message_queue_port=0
    l_failed_inputs_message_queue_ports=""

    validate_input_get_message_queue_port "API port" "$i_message_queue_port_api"
    if [ $? -ne 0 ]; then
        i_count_invalid_message_queue_port=`expr $i_count_invalid_message_queue_port + 1`
    fi

    if [ "X$i_message_queue_port_tcp" != "X5671" ]; then
        #
        # With 6.6.0, we are using the MQ as sealab queue. For this, we need to enable the TCP port
        # 5672 and TLS port 5671 by default when RPMs gets installed. These ports are mentioned in
        # /etc/rabbitmq/rabbitmq.conf file. Since, these gets opened by default when we install the
        # cyops-rabbitmq RPM, we need to add explicit skip here to not check the TCP port 5671 is aleady
        # in use or not when doing config vm of external sme.
        #
        validate_input_get_message_queue_port "TCP port" "$i_message_queue_port_tcp"
        if [ $? -ne 0 ]; then
            i_count_invalid_message_queue_port=`expr $i_count_invalid_message_queue_port + 1`
        fi
    fi

    # API and TCP port values cannot be the same
    # Should not give below error if both inputs are empty, hence added "$i_message_queue_port_api" non empty condition
    if [ ! -z "$i_message_queue_port_api" -a \
         "$i_message_queue_port_api" = "$i_message_queue_port_tcp" ]; then
        i_count_invalid_message_queue_port=`expr $i_count_invalid_message_queue_port + 1`
        l_failed_inputs_message_queue_ports="$l_failed_inputs_message_queue_ports\nPort numbers cannot be identical"
    fi

    if [ $i_count_invalid_message_queue_port -gt 0 ]; then
        if [ $flag_interactive_mode -eq 1 ]; then
            s_msg="$l_failed_inputs_message_queue_ports"
            i_dialog_height=`expr $i_count_invalid_message_queue_port + 6`
            i_dialog_width=60
            show_dialog_mbox "$s_title" \
                             "OK" \
                             "$s_msg" \
                             $i_dialog_height \
                             $i_dialog_width
        fi
        return 1
    fi
}

get_message_queue_ports() {
    i_message_queue_port_api=0
    i_message_queue_port_tcp=0
    if [ $flag_interactive_mode -eq 1 ]; then
        # Interactive mode
        s_dialog_form_text="\nSpecify the port numbers to be used for the API and TCP connections to the Secure Message Exchange Server:\n\nPlease note -\n1. Port numbers cannot be greater than 65535 or be empty or contain non-integer values\n2. API default port: $i_message_queue_port_default_api\n3. TCP default port: $i_message_queue_port_default_tcp"
        s_dialog_form_label_input1="API port:"
        s_dialog_form_value_input1=$i_message_queue_port_default_api
        s_dialog_form_label_input2="TCP port:"
        s_dialog_form_value_input2=$i_message_queue_port_default_tcp
        s_cancel_button_option="--no-cancel"
        s_form_type="--form"
        f_dialog_form_output="output.txt"
        i_dialog_form_edit_start=11
        i_dialog_form_edit_end=16
        while true; do
            show_dialog_form_get_two_inputs
            if [ $? -eq 0 ]; then
                # User pressed "OK"
                i_message_queue_port_api=$(cat $f_dialog_form_output | sed -n '1p' )
                i_message_queue_port_tcp=$(cat $f_dialog_form_output | sed -n '2p' )
                rm -f $f_dialog_form_output
                validate_input_get_message_queue_ports
                if [ $? -ne 0 ]; then
                    # Invalid input, so show the dialog again with the entered values
                    if [ -z "$i_message_queue_port_api" ]; then
                        s_dialog_form_value_input1=$i_message_queue_port_default_api
                    else
                        s_dialog_form_value_input1=$i_message_queue_port_api
                    fi
                    if [ -z "$i_message_queue_port_tcp" ]; then
                        s_dialog_form_value_input2=$i_message_queue_port_default_tcp
                    else
                        s_dialog_form_value_input2=$i_message_queue_port_tcp
                    fi
                    continue
                fi
                # Valid input, so exit the loop
                break
            fi
        done
    else
        # non interactive mode
        # message_queue_api_port and message_queue_tcp_port are sourced from the conf file
        if [ ! -z "$message_queue_api_port" ]; then
            # Found "message_queue_api_port=" in the conf
            i_message_queue_port_api=$message_queue_api_port
        fi

        if [ ! -z "$message_queue_tcp_port" ]; then
            # Found "message_queue_api_port=" in the conf
            i_message_queue_port_tcp=$message_queue_tcp_port
        fi

        if [ $i_message_queue_port_tcp -ne 0 -o $i_message_queue_port_api -ne 0 ]; then
            validate_input_get_message_queue_ports
            if [ $? -ne 0 ]; then
                l_failed_inputs="${l_failed_inputs}${l_failed_inputs_message_queue_ports}"
                flag_invalid_input=1
                return 1
            fi
        fi
    fi

    # Process the input if its non empty else skip
    if [ ! -z "$i_message_queue_port_api" ]; then
        s_msg="Set API port for messaging"
        s_cmd="$s_timeout_cmd csadm mq --configure-api-port $i_message_queue_port_api"
        add_to_execution_queue "SAPM" "$s_msg" "$s_cmd"
        add_to_summary "API Port" "$i_message_queue_port_api"
    else
        add_to_summary "API Port" "[Skipped]"
    fi

    if [ ! -z "$i_message_queue_port_tcp" ]; then
        s_msg="Set TCP port for messaging"
        s_cmd="$s_timeout_cmd csadm mq --configure-tcp-port $i_message_queue_port_tcp"
        add_to_execution_queue "STPM" "$s_msg" "$s_cmd"
        add_to_summary "TCP Port" "$i_message_queue_port_tcp"
    else
        add_to_summary "TCP Port" "[Skipped]"
    fi
}

configure_message_queue() {

    s_msg="Enable messaging management console"
    s_cmd="$s_timeout_cmd csadm mq --enable-console"
    add_to_execution_queue "EMMC" "$s_msg" "$s_cmd"

    s_msg="Enable messaging monitoring extension"
    s_cmd="$s_timeout_cmd csadm mq --enable-event-plugin"
    add_to_execution_queue "EMME" "$s_msg" "$s_cmd"

    s_msg="Enable messaging environment"
    s_cmd="$s_timeout_cmd csadm mq --enable-rabbitmq-env"
    add_to_execution_queue "EME" "$s_msg" "$s_cmd"

    get_message_queue_credentials

    get_message_queue_ports

    s_msg="Generate messaging virtual host"
    s_cmd="$s_timeout_cmd csadm mq vhost add --name $s_message_queue_vhost"
    add_to_execution_queue "GMVH" "$s_msg" "$s_cmd"

    # Delete the loopbackup user.
    sed -i "/loopback_users/d" $f_loopbackp
}

# Insert or replace the data in the file $f_ifcfg
insert_or_replace() {
    s_key=$1
    s_value=$2

    if [ -z "$s_value" ]; then
        # Optional entry
        return 0
    fi

    grep -q $s_key $f_ifcfg
    if [ $? -eq 0 ]; then
        sed -i "s/$s_key=\(.*\)/$s_key=$s_value/g" $f_ifcfg
    else
        echo "$s_key=$s_value" >> $f_ifcfg
    fi
}

update_invalid_ip_list() {
    s_address="$1"
    i_count_invalid_ip=`expr $i_count_invalid_ip + 1`
    if [ -z "$s_address" ]; then
        l_invalid_address=" - $s_address\n"
    else
        l_invalid_address="$l_invalid_address - $s_address\n"
    fi
}

validate_ip() {
    s_type=$1
    s_display="$2"
    s_ip="$3"

    # If its optional and empty, return 0
    # Ex: DNS2
    if [ "$s_type" = "optional" -a -z "$s_ip" ]; then
        return 0
    fi

    if [ -z "$s_ip" ]; then
        update_invalid_ip_list "$s_display"
        l_blank_ip_address="$l_blank_ip_address\n$s_display"
        flag_blank_entry=1
        # Empty IP field
        return 1
    fi

    if [[ ! "$s_ip" =~ $regex_ipv4 ]]; then
        update_invalid_ip_list "$s_display"
        return 1
    fi
}

validate_input_static_ip() {
    # Validate the IPs
    flag_blank_entry=0
    i_count_invalid_ip=0
    l_invalid_address=""
    l_blank_ip_address=""

    validate_ip mandatory "IP address" $s_static_ip

    validate_ip mandatory "Gateway address" $s_gateway

    validate_ip mandatory "Netmask address" $s_netmask

    validate_ip mandatory "DNS1 address" $s_dns1

    validate_ip optional "DNS2 address" $s_dns2

    if [ $i_count_invalid_ip -gt 0 ]; then
        # Show additional line in case any of the mandatory fields are left blank
        s_msg_blank=""
        i_blank_line_count=0
        if [ $flag_blank_entry -eq 1 ]; then
            s_msg_blank="\n(Fields marked by * cannot be left blank)"
            i_blank_line_count=1
        fi
        s_msg="\nInvalid data in below listed fields:$s_msg_blank\n$l_invalid_address\nPlease try again."
        i_dialog_height=`expr $i_count_invalid_ip + 8 + $i_blank_line_count`
        i_dialog_width=50
        show_dialog_mbox "$s_title" \
                         "OK" \
                         "$s_msg" \
                         $i_dialog_height \
                         $i_dialog_width
        return 1
    fi
}

update_ifcfg() {
    # Insert if not found, replace otherwise
    insert_or_replace IPADDR $s_static_ip

    insert_or_replace GATEWAY $s_gateway

    insert_or_replace NETMASK $s_netmask

    insert_or_replace DNS1 $s_dns1

    insert_or_replace DNS2 $s_dns2

    insert_or_replace BOOTPROTO none

    # VM has old mac address in ifcfg created at the time of VM generation
    # Once VM is imported, it gets new MAC
    # So update the ifcfg with new MAC adderss
    update_mac_address

    rpm -qi cloud-init > /dev/null
    if [ $? -eq 0 ];then
        # if cloud-init is used then disable the network config as it reassigns network using dhcp on reboot.
        disable_network_config_for_cloud_init
    fi
    
    systemctl restart NetworkManager
}

disable_network_config_for_cloud_init(){
    local f_cloud_network_cfg="$d_cloud_configs/99-custom-networking.cfg"
    echo "network:" > $f_cloud_network_cfg
    echo "  config: disabled" >> $f_cloud_network_cfg
}

get_static_ip() {
    if [ $flag_interactive_mode -eq 1 ]; then
        # Interactive mode
        s_dialog_form_title="Set static IP"
        s_dialog_form_text="\nPlease enter static IP details:\n(Fields marked with * are mandatory)"
        s_dialog_form_label_input1="Static IP* :"
        s_dialog_form_label_input2="Gateway*   :"
        s_dialog_form_label_input3="Netmask*   :"
        s_dialog_form_label_input4="DNS1*      :"
        s_dialog_form_label_input5="DNS2       :"
        f_dialog_form_output="output.txt"
        i_dialog_form_edit_start=14
        i_dialog_form_edit_end=21
        while true; do
            show_dialog_form_get_five_inputs
            s_static_ip=$(cat $f_dialog_form_output | sed -n '1p' )
            s_gateway=$(cat $f_dialog_form_output | sed -n '2p' )
            s_netmask=$(cat $f_dialog_form_output | sed -n '3p' )
            s_dns1=$(cat $f_dialog_form_output | sed -n '4p' )
            s_dns2=$(cat $f_dialog_form_output | sed -n '5p')
            rm -f $f_dialog_form_output
            validate_input_static_ip
            if [ $? -ne 0 ]; then
                # Invalid input
                s_dialog_form_value_input1=$s_static_ip
                s_dialog_form_value_input2=$s_gateway
                s_dialog_form_value_input3=$s_netmask
                s_dialog_form_value_input4=$s_dns1
                s_dialog_form_value_input5=$s_dns2
                continue
            fi
            # Valid input
            add_to_summary "Static IP" "$s_static_ip"
            add_to_summary "Gateway" "$s_gateway"
            add_to_summary "Netmask" "$s_netmask"
            add_to_summary "DNS1" "$s_dns1"
            if [ ! -z "$s_dns2" ]; then
                add_to_summary "DNS2" "$s_dns2"
            else
                add_to_summary "DNS2" "[Empty]"
            fi
            break
        done
    else
        # Non Interactive mode
        # static_ip, gateway, netmask, dns1 and dns2 are sourced from the conf file
        # Read static ip inputs only if static_ip field is present
        if [ ! -z "$static_ip" ]; then
            s_static_ip=$static_ip
            s_gateway=$gateway
            s_netmask=$netmask
            s_dns1=$dns1
            s_dns2=$dns2
            validate_input_static_ip
            if [ $? -ne 0 ]; then
                s_static_ip_error_text="\nWhile setting static IP address, please ensure below fields have valid IP addresses:\nstatic_ip\ngateway\nnetmask"
                l_failed_inputs="$l_failed_inputs\n$s_static_ip_error_text"
                flag_invalid_input=1
                return 1
            fi
        fi
    fi

    update_ifcfg
}

update_mac_address() {
    # Update the MAC address only if the address file is present
    if [ -f "$f_mac_address" ]; then
        # Replace the MAC address
        # Why: The MAC address ifcfg file currently has is of the VM from which the VAPP was created
        # Once the VAPP is imported, it gets new MAC. Need to replace this new MAC
        s_mac_address=`cat $f_mac_address`

        insert_or_replace HWADDR $s_mac_address
    fi
}

read_inputs() {
    f_config_vm_conf="$1"
    get_hostname

    if [ $flag_set_static_ip -eq 1 ]; then
        get_static_ip
    else
        get_dns
    fi

    if [ "$flag_is_enterprise" = "true" ]; then
        # Proxy Table is having a column with name proxies which is encrypted field.
        # Encrypted field relies on .Defuse key to be present on system prior to database entry.
        # Currently it was failing as the key was not present at time of proxy insertion at config vm level.
        s_msg="Generate encryption key"
        s_cmd="generate_encryption_key"
        add_to_execution_queue "GEK" "$s_msg" "$s_cmd"

        set_proxy
    elif [ "$flag_is_secure_message_exchange" = "true" ]; then
        # Message queue configuration if its SME
        configure_message_queue
    else
        echo "Error: Unrecognized instance type"
        my_exit 1
    fi
}

show_welcome_box() {
    s_title_welcome_box=$s_cust_title_welcome_box
    s_msg_welcome_box=$s_msg_cust_welcome_box
    show_dialog_mbox "$s_title_welcome_box" \
                     "OK" \
                     "$s_msg_welcome_box" \
                     $i_height_dialog \
                     $i_width_dialog
}

reset_variables() {
    s_summary=""
    s_cust_summary=""
    i_count_summary_lines=0
    truncate --size 0 $f_summary
    truncate --size 0 $f_config_vm_response
}

prepare_summary_message() {
    # Display proceed message on $i_proceed_msg_offset_y_axis th line
    #  from top of the dialog box
    i_proceed_msg_offset_y_axis=27

    # There are 5 lines from top till summary starts
    i_count_filler=`expr $i_proceed_msg_offset_y_axis - $i_count_summary_lines - 5`

    # Calculate number of empty lines between summary and proceed message
    s_empty_lines=$(printf '\\n%.0s' $(seq 1 $i_count_filler))

    while read line; do
        s_cust_summary="$s_cust_summary$line\n"
    done < "$f_summary"

    s_display_summary="\nReady to configure $s_config_type.\n
\n
Please review the inputs before proceeding for configuration.\n
\n
${s_cust_summary}${s_empty_lines}\n
Press \"Proceed\" to start configuration.\nPress \"Back\" to startover."
}

show_summary() {
    prepare_summary_message
    s_summary=$s_display_summary

    show_dialog_yesno "$s_title" \
                      "Back" \
                      "Proceed" \
                      "$s_summary" \
                      $i_height_dialog \
                      $i_width_dialog
    return $?
}

get_hardware_key() {
    set -x
    echo "Obtaining Device UUID"

    # We need backup of device uuid in flags dir specifically for FortiCloud VMs.
    # On FortiCloud, after the snapshot revert, the instance UUID changes. Hence to
    # know what was original instance UUID, we need backup. Please note that we have backup in
    # home folder, but since user can clean its home folder, we need it somewhere else.
    # Original instance UUID is required to quickly know the passwords in db_config.yml.
    #
    csadm license --get-device-uuid | tee $f_hkey $device_uuid_in_flags_dir >/dev/null
    echo "Obtaining Device UUID: Done"
    set +x
}

process_hardware_key() {
    get_hardware_key >> $f_log 2>&1 &
    i_pid_get_hardware_key=$!

    wait $i_pid_get_hardware_key
    i_ret_code_get_hardware_key=$?

    if [ 1 -eq $i_ret_code_get_hardware_key ]; then
        s_status="Failed"
        i_fail_flag=1
    else
        s_status="Done"
    fi
}

reset_password_mq() {
    set -e
    local plain_pass=$1
    local encrypted_pass=$2
    local db_config="/opt/cyops/configs/database/db_config.yml"
    local utils_py="$d_config_scripts/confUtil.py"

    # fsr-cluster user is getting used in HA, hence we need to update its password per instance.
    local mq_uname="fsr-cluster"
    local key_mq_password_in_db_config="mq_password"

    rabbitmqctl change_password $mq_uname $plain_pass
    python3 $utils_py -f $db_config -k $key_mq_password_in_db_config -v $encrypted_pass
}

reset_db_password() {
    set -e
    . /opt/cyops/adm/transaction.sh
    f_config=/opt/cyops/configs/database/db_config.yml
    f_conf_util=$d_config_scripts/confUtil.py

    # Generate password for PG
    i_hkey=`cat $f_hkey`
    s_passwd_cyberpgsql=$i_hkey
    # Set the newly generated password
    PGPASSWORD=$s_pg_pwd psql -U cyberpgsql -d venom -c "ALTER USER cyberpgsql WITH ENCRYPTED PASSWORD '$s_passwd_cyberpgsql';"
    PGPASSWORD=$s_pg_pwd psql -U cyberpgsql -d venom -c "ALTER USER postgres WITH ENCRYPTED PASSWORD '$s_passwd_cyberpgsql';"

    # Encrypt newly generated password
    s_passwd_cyberpgsql_encrypted=`python3 $d_config_scripts/manage_passwords.py --encrypt $s_passwd_cyberpgsql`

    # Update the db config with newly generated password
    python3 $f_conf_util -f $f_config -k pg_password -v $s_passwd_cyberpgsql_encrypted

    # Update the archival password with newly encrypted password.
    python3 $f_conf_util -f $f_config -k pg_archival_password -v $s_passwd_cyberpgsql_encrypted

    # enable elasticsearch security
    sh /opt/cyops-search/scripts/setup_elastic.sh $s_passwd_cyberpgsql "$s_passwd_cyberpgsql_encrypted" "fresh"

    # update the nodeid/clusterid in das.ini
    sh /opt/cyops-auth/scripts/update_nodeid_in_das_ini.sh

    # Reset the password for MQ fsr-cluster user
    reset_password_mq $s_passwd_cyberpgsql $s_passwd_cyberpgsql_encrypted

    update_pg_hba_conf_for_pg14

    # Clear API cache
    cd /opt/cyops-api
    sudo -u nginx php bin/console cache:clear --no-interaction

    # Please note that we are checking ES jdk in 'configure_default_ha_cluster'. This is NOT
    # related to HA but since we want some place to execute below code, added it here.
    #
    if [ ! -d "/usr/share/elasticsearch/jdk" ]; then
        return
    fi
    #
    # Remove the jdk which get ship with elasticsearch RPM. Please note that there is NO harm
    # in removing jdk that comes with elasticsearch RPM. Following line is mentioned on official
    # website of elasticsearch.
    # "The bundled JVM directory may be removed when using your own JVM."
    # https://www.elastic.co/guide/en/elasticsearch/reference/current/setup.html
    #
    rm -rf /usr/share/elasticsearch/jdk
}

update_pg_hba_conf_for_pg14(){
    #
    # From 7.2.2, postgresql upgrade from 12 to 14 and also password method change from trust to md5.
    # While installing the cyops-auth package it try to connect to database using password if md5 is already in \
    # pg_hba.conf file instead of trust. Also, we are setting the the postgresql password as hkey in configuration wizard.
    # So we are updating the pg_hba.conf file at the end of configuration wizard.
    # Same change also handled in upgrade script.
    # The use of the following variable is limited to the update pg_hba.conf only which is at the end of configuration wizard.
    # So declare the these variable to the following position.
    #
    local f_cmd_psql="/usr/bin/psql"
    local f_cmd_sed="/usr/bin/sed"
    local s_trust="trust"
    local s_md5="md5"
    local i_psql_version=$(${f_cmd_psql} --version | awk '{print $3}' | cut -d'.' -f1)
    local f_pg_hba_conf="/var/lib/pgsql/${i_psql_version}/data/pg_hba.conf"
    ${f_cmd_sed} -i "s/${s_trust}/${s_md5}/g" $f_pg_hba_conf
    /usr/bin/systemctl restart postgresql-${i_psql_version}
    sleep 5
}

configure_default_ha_cluster()
{
    # Bydefault, make the node as active primary for fresh install
    csadm ha join-cluster --role primary --status active
}

install_ansible() {
    s_version_ansible=7.4.0
    s_extra_index_url="--extra-index-url https://$product_yum_server/prod/connectors/deps/simple/"
    s_ansible_pip_cmd="sudo -u nginx $f_pip install ansible==$s_version_ansible $s_extra_index_url"
    a_ansible_services="celeryd celerybeatd uwsgi fsr-workflow"
    export LC_ALL="en_US.UTF-8"
    $s_timeout_cmd $s_ansible_pip_cmd
    if [ $? = 0 ]; then
        for sr in $a_ansible_services
        do
            systemctl restart $sr
            if [ $? != 0 ]; then
                echo "Failed to restart $sr service"
            else
                echo "Successfully restarted $sr service"
            fi
            sleep 5
        done
    else
        return 1
    fi
}

install_widgets() {
    sudo -u nginx php /opt/cyops-api/bin/console app:import:from:file -f /opt/cyops-api/src/DataFixtures/Widgets/widgets.json
}

sync_marketplace_metadata() {
    echo "Fetching latest marketplace metadata information."
    csadm package content-hub sync --force
    i_ret_code=$?
    if [ $i_ret_code -ne 0 ]; then
        echo "WARNING: Failed to sync marketplace metadata. Check proxy setup and other possible reasons for failure in the Troubleshooting section of the product documentation."
    fi
    # TODO : remove this change .temp fix for forticloud install issue.
    return 0
}

search_index() {
    echo "Hostname :" `hostname`
    sudo -u nginx php /opt/cyops-api/bin/console app:elastic:create
}

generate_encryption_key() {
    sudo php /opt/cyops-api/bin/generate-fsr-defuse-key >/dev/null
    chown nginx:nginx /opt/cyops/configs/cyops-api/.Defuse.key
    chmod 644 /opt/cyops/configs/cyops-api/.Defuse.key
}

generate_passwords_encryption_key(){
    python3 $d_config_scripts/manage_passwords.py --generate-encryption-key >/dev/null
    if [ $? -eq 1 ]; then
        echo "Failed to generate encryption key. Exiting config vm"
        my_exit 1
    fi
    chmod 644 /opt/cyops/configs/keys/PASSWORD_ENCRYPTION_KEY
}

# Add steps here which do not take any direct input from user
add_default_steps() {
    if [ "$flag_is_enterprise" = "true" ]; then
        s_msg="Update SSO certificate"
        s_cmd="sudo $s_timeout_cmd csadm certs --refresh-sso-cert"
        add_to_execution_queue "SSOC" "$s_msg" "$s_cmd"

        s_msg="Set up intra-service authentication"
        s_cmd="$s_timeout_cmd csadm certs --app-key"
        add_to_execution_queue "SISA" "$s_msg" "$s_cmd"

        s_msg="Generate certificates"
        s_cmd="$s_timeout_cmd csadm certs --generate $s_hostname"
        add_to_execution_queue "GC" "$s_msg" "$s_cmd"

        s_msg="Generate Device UUID"
        s_cmd="process_hardware_key"
        add_to_execution_queue "GDU" "$s_msg" "$s_cmd"

        s_msg="Reset database passwords"
        s_cmd="reset_db_password"
        add_to_execution_queue "RDP" "$s_msg" "$s_cmd"

        s_msg="Restart services"
        s_cmd="$s_timeout_cmd csadm services --restart"
        add_to_execution_queue "RS" "$s_msg" "$s_cmd"

        s_msg="Configure default single-node HA cluster"
        s_cmd="configure_default_ha_cluster"
        add_to_execution_queue "CDHC" "$s_msg" "$s_cmd"

        s_msg="Install python libraries"
        s_cmd="install_ansible"
        add_to_execution_queue "IPL" "$s_msg" "$s_cmd"

        s_msg="Sync marketplace metadata"
        s_cmd="sync_marketplace_metadata"
        add_to_execution_queue "IS" "$s_msg" "$s_cmd"

        s_msg="Install default widgets"
        s_cmd="install_widgets"
        add_to_execution_queue "IW" "$s_msg" "$s_cmd"

        s_msg="Search index initialization"
        s_cmd="search_index"
        add_to_execution_queue "SI" "$s_msg" "$s_cmd"

        s_msg="Refresh SSH keys"
        s_cmd="refresh_ssh_keys"
        add_to_execution_queue "RSK" "$s_msg" "$s_cmd"

        s_msg="Refresh data identifiers"
        s_cmd="refresh_uuids"
        add_to_execution_queue "RDI" "$s_msg" "$s_cmd"
    elif [ "$flag_is_secure_message_exchange" = "true" ]; then
        s_msg="Generate certificates"
        s_cmd="$s_timeout_cmd csadm mq certs generate"
        add_to_execution_queue "GC" "$s_msg" "$s_cmd"

        # Need to restart the rabbitmq-server service to reflect the changes.
        s_msg="Restart services"
        s_cmd="$s_timeout_cmd csadm services --restart"
        add_to_execution_queue "rbservice" "$s_msg" "$s_cmd"

        s_msg="Refresh SSH keys"
        s_cmd="refresh_ssh_keys"
        add_to_execution_queue "RSK" "$s_msg" "$s_cmd"
    fi
    s_msg="Post-configuration tasks"
    s_cmd="post_configuration_tasks"
    add_to_execution_queue "PCT" "$s_msg" "$s_cmd"
}

add_summary_to_log() {

    # Write the summary and the .response file into the log for assisting the debugging
    echo -e "\n\n$s_summary" >> $f_log
    echo -e "*******************************************************\n\n" >> $f_log
    echo "Contents of the $f_config_vm_response" >> $f_log
    cat "$f_config_vm_response" >> $f_log
    echo -e "*******************************************************\n\n" >> $f_log

    # Replace the password with *s
    sed -i -e "s/^\(.*http\(s\)\{0,1\}:\/\/.*:\)\(.*\)\(@.*\)$/\1\*\*\*\*\*\4/g" \
           -e "s/\(.*csadm network set-http.*--password \)\(.*\)/\1 \*\*\*\*\*/g" \
           $f_log
}

get_inputs() {
    # Write the PID
    echo $$ > "$f_pid_vm_config"

    if [ $flag_interactive_mode -eq 1 ]; then
        show_welcome_box
        while true; do
            reset_variables

            read_inputs

            show_summary
            if [ 1 -eq $? ]; then
                break
            fi
        done
    else
        # non-interactive mode
        reset_variables
        read_inputs "$f_config_vm_conf_default"
        if [ $flag_invalid_input -eq 1 ]; then
            echo -e "$l_failed_inputs"
            my_exit $flag_invalid_input
        fi
    fi

    add_default_steps

    # Write the summary and the .response file into the log for assisting the debugging
    add_summary_to_log
}

calculate_dots() {
    s_string=$1
    i_cols=$2
    i_dots=`expr $i_cols - ${#s_string}`
    s_v=$(printf "%-${i_dots}s" ".")
}

show_progress_update() {
    s_state="$1"
    calculate_dots "$s_command_display_text  [$s_status]" 55
    s_current_command_display_text="$s_command_display_text ${s_v// /.} [$s_status]"
    if [ "$s_state" = "in_progress" ]; then
        if [ $flag_interactive_mode -eq 1 ]; then
            s_command_complete_update_text="$s_command_execution_history\n  $s_current_command_display_text"
        else
            # for non interactive configuration and in progress task's status needs
            # to be updated on the same line, hence \r.
            # its not required for interactive config as in that case entire line
            # (previous tasks status and current) are redisplayed
            s_newline_char="\r"
        fi
    elif [ "$s_state" = "finished" ]; then
        if [ $flag_interactive_mode -eq 1 ]; then
            s_command_complete_update_text="$s_command_execution_history\n  $s_current_command_display_text"
            s_command_execution_history="$s_command_complete_update_text"
            i_steps_completed=`expr $i_steps_completed + 1`
        else
            s_newline_char="\n"
        fi
    fi

    if [ $flag_interactive_mode -eq 1 ]; then
        i_percentage=`expr $i_step_size \* $i_steps_completed`
        show_dialog_progress $i_percentage \
                             "$s_title" \
                             "$s_command_complete_update_text"
    else
        echo -ne "$s_current_command_display_text$s_newline_char"
    fi
}

run_command() {
    # Run command in background
    echo "Running: $s_command" >> $f_log 2>&1
    $s_command >> $f_log 2>&1 &

    i_pid_cmd=$!

    # Show the progress update
    s_status="In Progress"
    show_progress_update "in_progress"

    # Wait for command to finish
    wait $i_pid_cmd
    i_ret_code=$?
    if [ $i_ret_code -ne 0 ]; then
        s_status="Failed"
        i_fail_flag=1
    else
        s_status="Done"
    fi

    echo "*********************************************************************" >> $f_log 2>&1
    echo "" >> $f_log 2>&1

    # Show the progress update
    show_progress_update "finished"
}

show_final_message() {
    if [ $flag_interactive_mode -eq 1 ]; then
        s_title=$s_cust_title
        show_dialog_progress 100 \
                             "$s_title" \
                             "$s_command_execution_history"
        sleep 1
        show_dialog_mbox "$s_title" \
                         "$s_mbox_button_label" \
                         "$s_msg_config_finished" \
                         $i_height_dialog \
                         $i_width_dialog
    else
        echo -e "\n$s_abc
  Configuration log is saved at\n
  $f_log"
    fi
}

notify_ui_fortisoar_state() {
    if [[ "$flag_is_secure_message_exchange" == "true" && "$flag_is_enterprise" == "false" ]]; then
        return
    fi
    local payload=$1
    local file_fsr_provision="/opt/cyops-ui/fsr-boot.json"

    $f_fsr_state_updater update-state --app-name "provisioning" \
                       --state-file $f_fsr_state_json \
                       --payload "$payload"

    chmod 644 $file_fsr_provision
    chown nginx:nginx $file_fsr_provision
}

initialize_fsr_boot_file(){
    if [[ "$flag_is_secure_message_exchange" == "true" && "$flag_is_enterprise" == "false" ]]; then
        return
    fi
    $f_fsr_state_updater clear-key --app-name "provisioning" \
                         --state-file $f_fsr_state_json \
                         --key-name 'steps'
    # Notify init
    notify_ui_fortisoar_state  '{"provision": false,"state": "init","steps" : [], "force_proceed": true}'
}

configure_vm() {
    echo "Configuring FortiSOAR"
    initialize_fsr_boot_file

    # Calculate number of commands to be run during VM configuration
    # $f_config_vm_response has two lines for each command. Its display line and the command itself
    # So actual number of commands are total line count / 2
    i_steps_count=`expr $(wc -l $f_config_vm_response | awk '{print $1}') / 2`
    i_step_size=`expr 100 / $i_steps_count`
    i_max_percent=`expr $i_steps_count \* $i_step_size`
    i_steps_completed=0

    i_counter=1
    i_fail_flag=0

    for i_step_number in $(seq 1 $i_steps_count); do
        # From the pair of lines, first is the message and second is the command
        s_command_display_text=`sed -n "${i_counter}p" < $f_config_vm_response | cut -d: -f2`
        s_command=`sed -n "$(expr ${i_counter} + 1)p" < $f_config_vm_response | cut -d: -f2`

        run_command
        if [ $i_ret_code -eq 0 ]; then
            notify_ui_fortisoar_state "{\"steps\": [{\"label\" :\"$s_command_display_text\" , \"success\": true}]}"
        else
            notify_ui_fortisoar_state "{\"steps\": [{\"label\" :\"$s_command_display_text\" , \"success\": false}]}"
        fi
        # Jump by 2 because each command represents two lines (label and command) in the response file
        i_counter=`expr $i_counter + 2`
    done

    if [ 1 -eq $i_fail_flag ]; then
        notify_ui_fortisoar_state '{"provision": true, "state": "failed"}'
    else
        notify_ui_fortisoar_state '{"provision": true, "state": "provisioned"}'
    fi

    if [ 1 -eq $i_fail_flag ]; then
        s_abc="One or more tasks failed. Please check the configuration log for details.\n
  Please exit the wizard now, get the issues fixed and re-run the wizard by running\n
  below command:\n
      $0"
        # Check if ansible is installed, by running below command
        # If its not installed, set failed flag
        $f_pip show ansible
        if [ $? -ne 0 ]; then
            s_abc="$s_abc \n  Installation of ansible python library has failed. Some of the filter expression\n
  used in workflows might not work.\n
  This could be because of broken network connectivity to $product_yum_server.\n
  If the server connects via a proxy, ensure that the HTTPS_PROXY is set correctly\n
  using the csadm network command.\n
  Once the network connectivity is verified, run the following commands to reinstall\n
  the library:\n
  1. sudo -u nginx /opt/cyops-workflow/.env/bin/pip install ansible --extra-index-url\n
  https://$product_yum_server/prod/connectors/deps/simple/\n
  2. systemctl restart celeryd celerybeatd uwsgi"
            echo "$s_abc" >> $f_log
        fi
        touch $f_vm_configuration_failed
        s_msg_config_finished=" $s_command_execution_history\n\n  $s_abc\n
  Configuration log is saved at\n
  $f_log"
        show_dialog_mbox "$s_title" \
                         "Exit" \
                         "$s_msg_config_finished" \
                         $i_height_dialog \
                         $i_width_dialog
        my_exit 1
    else
        if [ ! -f $f_hkey ]; then
            # For Secure Message Exchange.
            s_abc="$s_config_type is now configured.\n
  For production usage, the following configurations are highly recommended:\n
  1. Replace the self-signed certificate on the server with a signed certificate from\n
  your organization using the following command:\n
      \Zb\Z4csadm mq certs deploy\n\Zn
  2. Mandate the client certificate verification(mTLS) and provide the Certificate\n
  Authority for the Client Certificates using the following command:\n
      \Zb\Z4csadm mq mtls enable\n
      csadm mq truststore add\n\Zn
  Refer to the product documentation for more details.\n\n
  Note: You will be logged out now for changes to take effect.\n
  Please log in again."

        else
            # For FortiSOAR
            i_hkey=`cat $f_hkey`
            s_abc="$s_config_type is now configured.\n
  Device UUID: $i_hkey(saved in $f_hkey)\n
  Please get license from Forticare using the Device UUID displayed above.\n
  Refer documentation for details.\n
  Note: You will be logged out now for changes to take effect.\n
  Please log in again.\n"
        fi
        touch $f_vm_configured $flag_vm_configured_in_flags_dir
        s_mbox_button_label="OK"
    fi
    s_msg_config_finished=" $s_command_execution_history\n\n
  $s_abc\n
  Configuration log is saved at\n
  $f_log"
    show_final_message
}

check_pid() {
    i_pid=$1
    ps -ef | grep -w $i_pid | grep -qv grep
    if [ 0 -eq $? ]; then
        return 0
    else
        return 1
    fi
}

show_dialog_wait_for_config_to_finish() {
    s_msg="$1"
    i_height_dialog_wait=3
    s_title=" Please Wait "
    i_msg_len=${#s_msg}
    i_width_dialog_wait=`expr $i_msg_len + 7`
    s_msg_show="$s_msg $i"
    if [ $flag_interactive_mode -eq 1 ]; then
        show_dialog_infobox "$s_title" \
                            "$s_msg" \
                            $i_height_dialog_wait \
                            $i_width_dialog_wait
    else
        echo -ne "$s_msg\r"
    fi
}

fix_logfiles_perms(){
    echo "Ensuring permissions on all logfiles are configured" >> $f_log 2>&1
    # Ensure permissions on all logfiles are configured
    find /var/log -type f -perm /g+wx,o+rwx -exec chmod --changes g-wx,o-rwx "{}" + >> $f_log 2>&1
}

disable_ipv6(){
    # Need to update the network interface config files too to disable IPv6
    sed -i "/^\s*IPV6.*/d" $f_ifcfg
    systemctl restart NetworkManager
}

post_configuration_tasks() {
    # For OS hardening
    fix_logfiles_perms
    disable_ipv6
    reserve_one_gb
    if [ "$flag_is_enterprise" = "true" ]; then
        listen_to_localhost
    fi
}

#
# Reserve the 1 GB space, in case disk full issue the user,
# will delete the file /var/lib/pgsql/resv_space and utilize the space
# when require and create the file again using the following command once issue resolve.
# The following command is dd if=/dev/zero of=/var/lib/pgsql/resv_space count=1024 bs=1048576
#
reserve_one_gb() {
    if [ "$flag_is_enterprise" = "true" ]; then

        dd if=/dev/zero of=$d_pgsql_path/$f_pgsql_resv_space_file count=1024 bs=1048576 >> $f_log 2>&1
        if [ $? -ne 0 ]; then
            echo "Error: Failed to create the $d_pgsql_path/$f_pgsql_resv_space_file file."
            echo "       Please make sure $d_pgsql_path has sufficient space and"
            echo "       re-run the following command manually:"
            echo "       dd if=/dev/zero of=$d_pgsql_path/$f_pgsql_resv_space_file count=1024 bs=1048576"
        fi
    fi
}

listen_to_localhost()  {
  # Update nginx listen configuration
  sh /opt/cyops/scripts/update_nginx_listen_config.sh
  fsr_docker_utils_sh="/opt/cyops/scripts/docker/utils.sh"
  source $fsr_docker_utils_sh
  if ! is_docker; then
    systemctl restart nginx
  fi
}

wait_for_config_to_finish() {
    while true; do
        i_wait_seconds=`expr $i_wait_minutes \* $i_seconds_per_minute`
        i_sleep=1
        while [ 0 -lt $i_wait_seconds ]; do
            if [ $i_wait_seconds -lt $i_seconds_per_minute ]; then
                i_display_time="$i_wait_seconds seconds"
            else
                i_display_time_minutes=`expr $i_wait_seconds / $i_seconds_per_minute`
                i_display_time_seconds=`expr $i_wait_seconds % $i_seconds_per_minute`
                i_display_time="$i_display_time_minutes:$i_display_time_seconds minutes"
            fi
            show_dialog_wait_for_config_to_finish "$s_config_type configuration is in progress in another session. Waiting for $i_display_time to finish"
            sleep $i_sleep
            i_wait_seconds=`expr $i_wait_seconds - $i_sleep`
            check_pid $i_pid_vm_config
            if [ 1 -eq $? ]; then
                if [ ! -f "$f_vm_configured" ]; then
                    # pid not found and .vm_configured also not found
                    # that means last session was timed out
                    # so configuration is not running
                    # So re-start configuration
                    show_config_timeout_mbox
                    return 0
                else
                    # pid not found and .vm_configured found
                    # Configuration has been successfully completed
                    # Login to command prompt
                    my_exit 0
                fi
            else
                # pid found
                continue
            fi
        done
        check_pid $i_pid_vm_config
        if [ 1 -eq $? ]; then
            if [ ! -f "$f_vm_configured" ]; then
                # pid not found and .vm_configured also not found
                # that means last session was timed out
                # so configuration is not running
                # So re-start configuration
                show_config_timeout_mbox
                return 0
            else
                # pid not found and .vm_configured found
                # Configuration has been successfully completed
                # Login to command prompt
                my_exit 0
            fi
        else
            # ask user to wait more or restart configuration
            s_msg="Looks like configuration is still running.\n
You have two options:\n
  1. Wait for $i_wait_minutes more minutes\n
  2. Restart the configuration [This will kill currently running configuration and start new one]"
            show_dialog_yesno "$s_title" \
                              "Wait for $i_wait_minutes more minutes" \
                              "Restart the configuration" \
                              "$s_msg" \
                              $i_height_dialog \
                              $i_width_dialog
            if [ 0 -eq $? ]; then
                # Wait for $i_wait_minutes more
                continue
            else
                # Kill currently running config and restart the config
                kill -9 $i_pid_vm_config > /dev/null 2>&1
                return 0
            fi
        fi
    done
}

show_config_timeout_mbox() {
    show_one_liner_mbox "$s_config_type configuration wizard was timed out in another session. Restarting $s_config_type configuration."
}

check_prev_failure() {
    if [ -f $f_vm_configuration_failed ]; then
        s_warning_msg="$s_config_type configuration failed in last run."
        if [ $flag_interactive_mode -eq 1 ]; then
            s_warning_msg_interactive="\n\n$s_warning_msg\n
    \n\n\n\n\n\n\n\n
    Press \"Proceed\" to configure $s_config_type, if issues are fixed.\n
    Press \"Exit\" to exit the wizard."
             show_dialog_yesno "$s_title" \
                               "Exit" \
                               "Proceed" \
                               "$s_warning_msg_interactive" \
                               $i_height_dialog \
                               $i_width_dialog
            if [ 0 -eq $? ]; then
                my_exit 0
            fi
            rm -f $f_vm_configuration_failed
            return 0
        else
            # Non-interactive mode
            echo -e "$s_warning_msg\n"
            echo -e "Please:"
            echo -e "1. Check the config vm log file"
            echo -e "2. Fix the problems"
            echo -e "3. Remove $f_vm_configuration_failed"
            echo -e "4. Re-run the configuration either by closing this ssh session and re-logging in or by running $d_config_scripts/config-vm.sh"
            exit
        fi
    else
        return 2
    fi
}

check_for_another_vm_config_session() {
    check_prev_failure
    if [ 0 -eq $? ]; then
        # If user chooses to reconfigure, skip further checks
        return 0
    fi

    if [ ! -f "$f_vm_configured" ]; then

        if [ -f "$f_pid_vm_config" ]; then
            # pid file found
            # Double check to see if the pid in the file too exists
            i_pid_vm_config=`cat $f_pid_vm_config`
            check_pid $i_pid_vm_config
            if [ 0 -eq $? ]; then
                # .vm_configured file not found
                # pid file found and valid pid too is found
                # Configuration is really running
                wait_for_config_to_finish
                return $?
            else
                # .vm_configured file not found
                # pid file found but valid pid not found
                # that means last session was timed out
                # so configuration is not running
                # So re-start configuration
                show_config_timeout_mbox
                return 0
            fi
        else
            # pid file not found and .vm_configured file not found
            # So good to start with configuration
            return 0
        fi
    else
        # .vm_configured file found
        # Configuration has been successfully completed
        # Login to command prompt
        my_exit 0
    fi
}

show_services_status_progress() {
    s_state="$1"
    calculate_dots "$s_command_display_text  [$s_status]" 73
    s_current_command_display_text="$s_command_display_text ${s_v// /.} [$s_status]"
    s_title_service_status="Preparing wizard"

    if [ "$s_state" = "in_progress" ]; then
        if [ $flag_interactive_mode -eq 1 ];then
            show_dialog_infobox "$s_title_service_status" "\n  $s_current_command_display_text" 9 82
            sleep 2
        else
            echo "$s_current_command_display_text"
        fi
    elif [ "$s_state" = "finished" ]; then
            if [ "$s_status" == "Failed" ];then
                s_common_msg="Refer to the document below for troubleshooting:"
                url_troubleshoot_doc="https://docs.fortinet.com/document/fortisoar/7.4.2/deployment-guide/"
                
                if [ $flag_interactive_mode -eq 1 ];then
                    s_current_command_display_text="\n  
  $s_current_command_display_text\n\n
  $s_common_msg \n
    $url_troubleshoot_doc \n\n  
  Exiting the configuration wizard."
                    show_dialog_mbox "$s_title_service_status" "Exit" "$s_current_command_display_text" 11 82
                    my_exit 0
                else
                    echo -ne "\033[A\033[2K"
                    echo -e "$s_current_command_display_text"
                    echo -e "$s_common_msg"
                    echo -e "    $url_troubleshoot_doc"
                    my_exit 0
                fi
            else
                if [ $flag_interactive_mode -eq 1 ];then
                    show_dialog_infobox "$s_title_service_status" "\n  $s_current_command_display_text" 9 82
                    sleep 2
                else
                    # Clear above line and write new statement on console
                    echo -ne "\033[A\033[2K"
                    echo -e "$s_current_command_display_text"
                fi
            fi
    fi
}

# Checking which services are down and writing flag 
# and services in files as this function runs in a subshell.
check_essential_services_status(){
    l_all_down_services=$(sudo csadm services --status | grep 'Not Running' | cut -d'.' -f1)
    
    # Check if any services is not running ( l_all_down_services is not null )
    if [ -n "$l_all_down_services" ]; then
        check_down_service_is_essential "$l_all_down_services"

       # Check if essential services is not running 
        if [ -n "$l_essential_down_services" ]; then
        # One or More essential service is not running 
        # So need to start those services
            sudo systemctl start $l_essential_down_services > /dev/null 2>&1
            l_down_services_after_start=$(sudo csadm services --status | grep 'Not Running' | cut -d'.' -f1)
            
            # check if services are not running even after start.
            if [ -n "$l_down_services_after_start" ]; then
                check_down_service_is_essential "$l_down_services_after_start"

                # Check if after start essential service is running
                if [ -n "$l_essential_down_services" ]; then
                    # flag_if_essential_services_up="false"
                    echo "false" > $f_flag_if_essential_services_up
                    echo "$l_essential_down_services" > $f_essential_down_services
                    return
                fi
            fi
        fi
    fi
    # flag_if_essential_services_up="true"
    echo "true" > $f_flag_if_essential_services_up
}

# Checking if down services are from essential service 
# and returning value in $l_essential_down_services
check_down_service_is_essential(){
    l_essential_services="nginx \
                        postgresql-14 \
                        rabbitmq-server \
                        cyops-auth \
                        php-fpm"

    l_essential_down_services=""
    l_down_services=$1

    for s_service in $l_down_services; do
            # Check if the s_service is in l_essential_services
            if [[ " $l_essential_services " =~ " $s_service " ]]; then
                l_essential_down_services="$l_essential_down_services $s_service"
            fi
    done
}

# running function in background and showing status and messages 
# as per failed or successfull in dialog box
check_service_status(){
    check_essential_services_status &

    i_pid_cmd=$!

    s_status="In Progress"
    s_command_display_text="Verifying prerequisites for the configuration workflow"

    show_services_status_progress "in_progress"
    
    wait $i_pid_cmd

    flag_if_essential_services_up=$(cat $f_flag_if_essential_services_up)
    sudo rm -f $f_flag_if_essential_services_up

    if [ "$flag_if_essential_services_up" == "false" ]; then
        s_status="Failed"
        l_essential_down_services=$(cat $f_essential_down_services)
        sudo rm -f $f_essential_down_services
    else
        s_status="Done"
    fi

    show_services_status_progress "finished"
}

prechecks() {
    # Exit if the product is already configured
    if [ -f $f_vm_configured ]; then
        show_one_liner_mbox "$s_config_type is already configured. Exiting the configuration wizard."

        # During inplace upgrade, if user logs in via SSH just after cyops-common rpm installation,
        # .bash_profile has config-vm call and control comes here which kills all csadmin sessions.
        # So workaround for 730 is to not to call my_exit
        # Since init function would have been already disabled the ctrl c, need to separately enable it back as my_exit is no longer getting called from here which has trap 2 call
        # https://mantis.fortinet.com/bug_view_page.php?bug_id=0871205
        trap 2
        exit 0
    fi

    check_service_status

    if [ $flag_interactive_mode -eq 0 ]; then
        # Non interactive mode
        # Source the default conf file
        chmod +x $f_config_vm_conf_default
        . $f_config_vm_conf_default
        if [ ! -z "$f_config_vm_conf_custom" ]; then
            # User provided custom conf option
            if [ ! -f "$f_config_vm_conf_custom" ]; then
                # Custom conf file does not exist
                echo "Error: $f_config_vm_conf_custom not found"
                my_exit 1
            fi

            # Get the first char to check if parameter path is absolute
            c_first_char=${f_config_vm_conf_custom:0:1}
            if [ "$c_first_char" = "/" ]; then
                # Custom conf file path should be absolute and the reason is below
                # If relative path is provided, the shell looks for
                # the file to source into $PATH, which is not what is intended

                # Source the custom conf file
                chmod +x $f_config_vm_conf_custom
                . $f_config_vm_conf_custom
            else
                echo "Error: Please provide absolute path of the custom configuration file"
                my_exit 1
            fi
        fi
    fi

    check_for_another_vm_config_session
}

init() {

    # State updater files for UI notification
    f_fsr_state_json="/opt/cyops-ui/fsr-boot.json"
    f_fsr_state_updater="/opt/cyops/scripts/state-updater.py"
    d_config_scripts="/opt/cyops/scripts"

    # Generate encryption key
    generate_passwords_encryption_key

    . /opt/cyops/adm/transaction.sh
    # Initializations

    # variables in /etc/environment are not available when this script is run in docker or via cloud-init

    # hence need to source explicitly
    source /etc/environment

    # Disable control C
    trap "" SIGINT

    # Fix for garbeled chars showing up in putty
    export NCURSES_NO_UTF8_ACS=1

    i_height_dialog=35
    i_width_dialog=90

    i_height_form=20
    i_width_form=60

    i_height_inputbox=13
    i_width_inputbox=60

    i_spinner_delay=0.5

    i_count_summary_lines=0

    d_home_csadmin="/home/csadmin"

    f_flag_if_essential_services_up="$d_home_csadmin/flag_if_essential_services_up.txt"
    f_essential_down_services="$d_home_csadmin/essential_down_services.txt"
    flag_if_essential_services_up="false"

    f_csadm_suggestor=/opt/cyops/scripts/csadm/auto_completion/csadm_auto_completion_suggestor.py
    export all_names_for_current_flavor=`$f_csadm_suggestor --get-all-names-for-current-flavor`

    f_script_name=`basename $0`
    f_pid_vm_config=$d_home_csadmin/${f_script_name}.pid
    f_script_name="${f_script_name%.*}"
    f_etc_environment="/etc/environment"
    # Removes pid file upon exit
    trap "rm -f -- '$f_pid_vm_config'" EXIT

    i_wait_minutes=10
    i_seconds_per_minute=60
    i_default_timeout_seconds=900

    i_password_length_mininum=8

    flag_invalid_input=0

    i_hkey=""

    # Message queue parameters
    s_message_queue_default_user="guest"
    s_message_queue_vhost="cyops-admin"
    i_message_queue_port_max=65535
    i_message_queue_port_default_api=15671
    i_message_queue_port_default_tcp=5671

    f_loopbackp="/etc/rabbitmq/rabbitmq.conf"

    f_pip="/opt/cyops-workflow/.env/bin/pip"

    s_cmd_message_queue_user_add="csadm mq user add"

    s_timeout_cmd="timeout --foreground $i_default_timeout_seconds"

    l_failed_inputs=""

    f_dialog_cmd=dialog

    d_pgsql_path="/var/lib/pgsql"
    f_pgsql_resv_space_file="resv_space"

    d_cloud_configs="/etc/cloud/cloud.cfg.d/"
    f_summary="$d_home_csadmin/summary.conf"
    
    f_config_vm_response="$d_home_csadmin/config-vm.response"
    f_config_vm_conf_default="$d_config_scripts/config-vm-enterprise-default.conf"

    f_hkey="$d_home_csadmin/device_uuid"
    flags_dir="/opt/cyops/configs/.flags"
    device_uuid_in_flags_dir="$flags_dir/.device-uuid"
    flag_vm_configured_in_flags_dir="$flags_dir/.vm_configured"
    f_vm_configured=$d_home_csadmin/.vm_configured
    f_vm_configuration_failed=$d_home_csadmin/.vm_configuration_failed

    f_bash_profile="$d_home_csadmin/.bash_profile"
    s_my_date=`date +%d_%b_%Y_%Hh_%Mm_%Ss`
    f_log=/var/log/cyops/install/config-vm-${s_my_date}.log

    regex_hostname="(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])"
    regex_ipv4="(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])"
    regex_ipv6="((([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))|\[(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))\])"
    regex_username="[a-z_][a-z0-9_-]*[$]?"

    # $regex_url does not have https? because same regex is used to match either http or https based on function parameter
    # A separate $s_protocol variable is prepended
    # Please check function extract_values "else" block
    regex_url="http?:\/\/($regex_username:.+@)?($regex_ipv4|$regex_ipv6|$regex_hostname):[0-9]{1,5}$"

    regex_noproxy="^($regex_ipv4|$regex_ipv6|$regex_hostname)(,($regex_ipv4|$regex_ipv6|$regex_hostname))*$"

    # Find the interface name
    s_network_interface=`ls -d1 /sys/class/net/* | grep -v lo | xargs basename`
    f_ifcfg="/etc/sysconfig/network-scripts/ifcfg-$s_network_interface"
    f_mac_address="/sys/class/net/$s_network_interface/address"

    # Check if IP is assigned to this VM
    # If yes, set the flag and step label accordingly
    flag_set_static_ip=0
    s_step_two=" 2. Get DNS"
    s_cur_ip=`hostname -I`
    if [ -z "$s_cur_ip" ]; then
        flag_set_static_ip=1
        s_step_two=" 2. Set Static IP"
    fi

    # Set the instance type flag
    is_secure_message_exchange_instance
    is_enterprise_instance

    # TBD: Need to fix repeatitive variable assignment, variable names below
    s_config_type="FortiSOAR"
    s_backtitle="$s_config_type Configuration"
    s_title="$s_config_type Configuration"
    s_cust_title="Configuring $s_config_type"
    s_cust_title_welcome_box="Welcome to $s_config_type Configuration Wizard"

    if [ "$flag_is_enterprise" = "true" ]; then
            s_msg_cust_welcome_box="\n
This wizard helps you to perform one-time FortiSOAR configuration.\n\n
Following tasks will be carried out as part of configuration:\n
 1. Change hostname and refresh MQ nodename [Optional]\n
$s_step_two\n
 3. Update network configuration [Optional]\n
 4. Set up intra-service authentication\n
 5. Generate certificates\n
 6. Generate Device UUID\n
 7. Reset database passwords\n
 8. Restart services\n
 9. Configure default single-node HA cluster\n
10. Configure proxy\n
11. Install python libraries\n
12. Sync marketplace metadata\n
13. Install default widgets\n
14. Search index initialization\n
15. Refresh SSH keys\n
16. Generate default encryption keys\n
17. Refresh data identifiers\n
\n\n\n\n\n\n\n\nPress \"OK\" to proceed."
    elif [ "$flag_is_secure_message_exchange" = "true" ]; then
        s_config_type="FortiSOAR Secure Message Exchange"
        s_backtitle="$s_config_type Configuration"
        s_title="$s_config_type Configuration"
        s_cust_title="Configuring $s_config_type"
        s_cust_title_welcome_box="Welcome to $s_config_type Configuration Wizard"

        f_config_vm_conf_default="$d_config_scripts/config-vm-sme-default.conf"

        s_msg_cust_welcome_box="\n
This wizard helps you to perform one-time $s_config_type configuration.\n
Following tasks will be carried out as part of configuration:\n
 1. Change hostname and refresh MQ nodename [Optional]\n
$s_step_two\n
 3. Enable messaging environment\n
 4. Generate messaging credentials\n
 5. Set new port for messaging\n
 6. Set new tcp port for messaging\n
 7. Generate messaging virtual host\n
 8. Enable messaging management console\n
 9. Generate certificates\n
10. Refresh SSH keys\n
\n\n\n\n\nPress \"OK\" to proceed."
    fi
}

main() {
    OPTS=`getopt -o ab: --long non-interactive,conf: -n "autoconf" -- "$@"`

    eval set -- "$OPTS"

    flag_interactive_mode=1
    while true; do
        case "$1" in
            -a|--non-interactive)
                export flag_interactive_mode=0
                shift
            ;;

            -b|--conf)
                f_config_vm_conf_custom=$2
                shift 2
            ;;

            -- ) shift
                 break ;;

            * ) break ;;
        esac
    done

    # All initialisations, hardcodings go here
    init

    prechecks

    get_inputs

    configure_vm

    my_exit 0
}

########
# MAIN #
########
main $*
