#!/usr/bin/env python3
""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import os, argparse, sys

if os.getuid() != 0:
    print("This script must be run as 'root'")
    exit(1)

from fsr.commonlibs.utils import CustomHelpFormatter

sys.path.append("/opt/cyops/scripts/")

from csadm.models.rsyslog import handle_rsyslog_operations
from csadm.consts import *
from csadm.message_constants import *

def check_if_ca_cert_required():
    required = False
    if ('--tls' in sys.argv) and (PROTOCOL_TCP in sys.argv):
        for validator in TLS_YES_CHOICES:
            if validator in sys.argv:
                required = True
                break
    return required

def _add_add_rsyslog_server_subparser(sub_parser_rsyslog):

    parser_add_rsyslog = sub_parser_rsyslog.add_parser('add-config', help=ADD_SERVER_HELP)
    parser_add_rsyslog.set_defaults(operation='rsyslog-add-server')

    parser_add_rsyslog.add_argument('--server',
                                    help=INPUT_ARGS_SERVER_IP_HELP,
                                    dest=INPUT_ARGS_SERVER_IP,
                                    required=True)

    parser_add_rsyslog.add_argument('--port',
                                    help=INPUT_ARGS_SERVER_PORT_HELP,
                                    dest=INPUT_ARGS_SERVER_PORT,
                                    required=True, type=int)

    parser_add_rsyslog.add_argument('--protocol',
                                    action='store',
                                    choices=PROTOCOL_CHOICES,
                                    type=str,
                                    help=INPUT_ARGS_SERVER_PROTOCOL_HELP,
                                    dest=INPUT_ARGS_SERVER_PROTOCOL,
                                    required=True)

    parser_add_rsyslog.add_argument('--tls',
                                    action='store',
                                    choices=TLS_CHOICES,
                                    type=str,
                                    help=INPUT_ARGS_SERVER_TLS_HELP,
                                    dest=INPUT_ARGS_SERVER_TLS,
                                    default=DEFAULT_TLS,
                                    required=False)

    parser_add_rsyslog.add_argument('--ca-cert',
                                    type=str,
                                    help=INPUT_ARGS_CA_CERT_HELP,
                                    dest=INPUT_ARGS_CA_CERT,
                                    default=None,
                                    required=check_if_ca_cert_required())

    parser_add_rsyslog.add_argument('--client-cert',
                                    type=str,
                                    help=INPUT_ARGS_CLIENT_CERT_HELP,
                                    dest=INPUT_ARGS_CLIENT_CERT,
                                    required=False,
                                    default=None)

    parser_add_rsyslog.add_argument('--client-key',
                                    type=str,
                                    help=INPUT_ARGS_CLIENT_KEY_HELP,
                                    dest=INPUT_ARGS_CLIENT_KEY,
                                    required='--client-cert' in sys.argv,
                                    default=None)

    parser_add_rsyslog.add_argument('--filter',
                                    type=str,
                                    help=INPUT_ARGS_FILTER_HELP,
                                    dest=INPUT_ARGS_FILTER,
                                    required=False,
                                    default=DEFAULT_FILTER)

    parser_add_rsyslog.add_argument('--config-name',
                                    type=str,
                                    help=INPUT_ARGS_CONFIG_NAME_HELP,
                                    dest=INPUT_ARGS_CONFIG_NAME,
                                    required=True)

    parser_add_rsyslog.add_argument('--uuid',
                                    type=str,
                                    dest=INPUT_ARGS_UUID,
                                    default= None,
                                    help=argparse.SUPPRESS,
                                    required=False)

    parser_add_rsyslog.add_argument('--no-interaction',
                                    action='store_false',
                                    dest=INPUT_ARGS_NO_INTERACTION,
                                    default=True,
                                    required=False,
                                    help=INPUT_ARGS_NO_INTERACTION_HELP)

    parser_add_rsyslog.add_argument('--source',
                                    action='store',
                                    choices=CMD_SOURCE_ALL,
                                    type=str,
                                    help=INPUT_ARGS_CMD_SOURCE_HELP,
                                    dest=INPUT_ARGS_CMD_SOURCE,
                                    default=CMD_SOURCE_DEFAULT,
                                    required=False)

def _add_show_rsyslog_server_subparser(sub_parser_rsyslog):
    parser_show_config_rsyslog = sub_parser_rsyslog.add_parser('show-config', help=SHOW_CONFIG_HELP)
    parser_show_config_rsyslog.set_defaults(operation='rsyslog-show-config')

    parser_show_config_rsyslog.add_argument('--json', action='store_true', dest=INPUT_ARGS_JSON_FLAG, default=False,
                                    required=False,
                                    help=INPUT_ARGS_JSON_FLAG_HELP)

    parser_show_config_rsyslog.add_argument('--uuid',
                                              type=str,
                                              help=INPUT_ARGS_UUID_HELP,
                                              dest=INPUT_ARGS_UUID,
                                              default=None,
                                              required=False)

    parser_show_config_rsyslog.add_argument('--source',
                                            action='store',
                                            choices=CMD_SOURCE_ALL,
                                            type=str,
                                            help=INPUT_ARGS_CMD_SOURCE_HELP,
                                            dest=INPUT_ARGS_CMD_SOURCE,
                                            default=CMD_SOURCE_DEFAULT,
                                            required=False)


def _add_remove_rsyslog_server_subparser(sub_parser_rsyslog):
    parser_remove_server_rsyslog = sub_parser_rsyslog.add_parser('remove-config', help=REMOVE_SERVER_HELP)
    parser_remove_server_rsyslog.set_defaults(operation='rsyslog-remove-server')

    parser_remove_server_rsyslog.add_argument('--uuid',
                                    type=str,
                                    help=INPUT_ARGS_UUID_HELP,
                                    dest=INPUT_ARGS_UUID,
                                    required=True)

    parser_remove_server_rsyslog.add_argument('--source',
                                    action='store',
                                    choices=CMD_SOURCE_ALL,
                                    type=str,
                                    help=INPUT_ARGS_CMD_SOURCE_HELP,
                                    dest=INPUT_ARGS_CMD_SOURCE,
                                    default=CMD_SOURCE_DEFAULT,
                                    required=False)

def _add_update_rsyslog_server_subparser(sub_parser_rsyslog):

    parser_update_rsyslog = sub_parser_rsyslog.add_parser('update-config', help=UPDATE_CONFIG_HELP)
    parser_update_rsyslog.set_defaults(operation='rsyslog-update-server')

    # For update-config, make sure you set default value as None for all parameters.
    parser_update_rsyslog.add_argument('--server',
                                    help=INPUT_ARGS_SERVER_IP_HELP,
                                    dest=INPUT_ARGS_SERVER_IP,
                                    required=False)

    parser_update_rsyslog.add_argument('--port',
                                    help=INPUT_ARGS_SERVER_PORT_HELP,
                                    dest=INPUT_ARGS_SERVER_PORT,
                                    required=False,
                                    type=int)

    parser_update_rsyslog.add_argument('--protocol',
                                    action='store',
                                    choices=PROTOCOL_CHOICES,
                                    type=str,
                                    help=INPUT_ARGS_SERVER_PROTOCOL_HELP,
                                    dest=INPUT_ARGS_SERVER_PROTOCOL,
                                    required=False)

    parser_update_rsyslog.add_argument('--tls',
                                    action='store',
                                    choices=TLS_CHOICES,
                                    type=str,
                                    help=INPUT_ARGS_SERVER_TLS_HELP,
                                    dest=INPUT_ARGS_SERVER_TLS,
                                    default=None,
                                    required=False)

    parser_update_rsyslog.add_argument('--ca-cert',
                                    type=str,
                                    help=INPUT_ARGS_CA_CERT_HELP,
                                    dest=INPUT_ARGS_CA_CERT,
                                    default=None,
                                    required=check_if_ca_cert_required())

    parser_update_rsyslog.add_argument('--client-cert',
                                    type=str,
                                    help=INPUT_ARGS_CLIENT_CERT_HELP,
                                    dest=INPUT_ARGS_CLIENT_CERT,
                                    required=False,
                                    default=None)

    parser_update_rsyslog.add_argument('--client-key',
                                    type=str,
                                    help=INPUT_ARGS_CLIENT_KEY_HELP,
                                    dest=INPUT_ARGS_CLIENT_KEY,
                                    required='--client-cert' in sys.argv,
                                    default=None)

    parser_update_rsyslog.add_argument('--filter',
                                    type=str,
                                    help=INPUT_ARGS_FILTER_HELP,
                                    dest=INPUT_ARGS_FILTER,
                                    required=False,
                                    default=None)

    parser_update_rsyslog.add_argument('--config-name',
                                    type=str,
                                    help=INPUT_ARGS_CONFIG_NAME_HELP,
                                    dest=INPUT_ARGS_CONFIG_NAME,
                                    required=False)

    parser_update_rsyslog.add_argument('--uuid',
                                       type=str,
                                       help=INPUT_ARGS_UUID_HELP,
                                       dest=INPUT_ARGS_UUID,
                                       required=True)

    parser_update_rsyslog.add_argument('--source',
                                        action='store',
                                        choices=CMD_SOURCE_ALL,
                                        type=str,
                                        help=INPUT_ARGS_CMD_SOURCE_HELP,
                                        dest=INPUT_ARGS_CMD_SOURCE,
                                        default=None,
                                        required=False)

def _add_import_config_subparser(sub_parser_rsyslog):
    parser_add_config_from_json = sub_parser_rsyslog.add_parser('import-config')
    parser_add_config_from_json.set_defaults(operation='rsyslog-import-config')
    parser_add_config_from_json.add_argument('--payload',
                                             action='store',
                                             type=str,
                                             dest=INPUT_ARGS_PAYLOAD,
                                             help=INPUT_ARGS_PAYLOAD_HELP,
                                             default=None,
                                             required=True)

    parser_add_config_from_json.add_argument('--no-interaction',
                                    action='store_false',
                                    dest=INPUT_ARGS_NO_INTERACTION,
                                    default=True,
                                    required=False,
                                    help=INPUT_ARGS_NO_INTERACTION_HELP)

def _add_export_config_subparser(sub_parser_rsyslog):
    parser_export_config = sub_parser_rsyslog.add_parser('export-config')
    parser_export_config.set_defaults(operation='rsyslog-export-config')

    parser_export_config.add_argument('--export-path',
                                      nargs='?',
                                      action='store',
                                      type=str,
                                      dest=INPUT_ARGS_EXPORT_PATH,
                                      required=False,
                                      default=None,
                                      help=INPUT_ARGS_EXPORT_PATH_HELP,
                                      const="dummy_value")

    parser_export_config.add_argument('--format',
                                    action='store',
                                    choices=EXPORT_FORMAT_CHOICES,
                                    type=str,
                                    help=INPUT_ARGS_EXPORT_FORMAT_HELP,
                                    dest=INPUT_ARGS_EXPORT_FORMAT,
                                    required=False,
                                    default=DEFAULT_EXPORT_FORMAT)

def _add_refresh_config_subparser(sub_parser_rsyslog):
    parser_refresh_config = sub_parser_rsyslog.add_parser('refresh-config')
    parser_refresh_config.set_defaults(operation='rsyslog-refresh-config')

def add_rsyslog_subparser(subparsers):
    parser_rsyslog = subparsers.add_parser('forward', help=LOG_FORWARDING_FEATURE_HELP)
    visible_options = "{add-config,show-config,remove-config,update-config}"
    sub_parser_rsyslog = parser_rsyslog.add_subparsers(metavar=visible_options)
    _add_add_rsyslog_server_subparser(sub_parser_rsyslog)
    _add_show_rsyslog_server_subparser(sub_parser_rsyslog)
    _add_remove_rsyslog_server_subparser(sub_parser_rsyslog)
    _add_update_rsyslog_server_subparser(sub_parser_rsyslog)
    _add_export_config_subparser(sub_parser_rsyslog)
    _add_import_config_subparser(sub_parser_rsyslog)
    _add_refresh_config_subparser(sub_parser_rsyslog)

    for name, subp in sub_parser_rsyslog.choices.items():
        subp._positionals.title = 'subcommand options are'
        subp._optionals.title = 'subcommand options are'


def define_parser(parser):
    visible_options = "{forward}"
    parser.add_argument('--collect', nargs='?', action='store', type=str, dest=INPUT_ARGS_LOG_PATH,
                       help=INPUT_ARGS_LOG_PATH_HELP, required=False, default=None, const="__placeholder_value__")

    parser.add_argument('--password',
                        type=str,
                        nargs='?',
                        dest=INPUT_ARGS_LOG_FILE_PASSWORD,
                        help=INPUT_ARGS_LOG_FILE_PASSWORD_HELP,
                        required=False,
                        default=None,
                        const="__placeholder_value__")

    subparsers = parser.add_subparsers(metavar=visible_options)
    add_rsyslog_subparser(subparsers)

    for name, subp in subparsers.choices.items():
        subp._positionals.title = 'subcommand options are'
        subp._optionals.title = 'subcommand options are'


def handle_log_collect():
    cmd = LOG_COLLECT_SCRIPT + " " + " ".join(sys.argv[1:]) + " 2>&1 | tee --append " + RSYSLOG_LOG_FILE
    ret = os.system("set -o pipefail; " + cmd)
    if ret != 0:
        sys.exit(1)
    sys.exit(0)

def start():
    parser = argparse.ArgumentParser(prog="csadm log", formatter_class=CustomHelpFormatter)
    parser._positionals.title = 'subcommand options are'
    parser._optionals.title = 'subcommand options are'

    define_parser(parser)

    args = parser.parse_args()
    input_args = vars(args)

    if input_args[INPUT_ARGS_LOG_PATH] is not None:
        handle_log_collect()

    if not 'operation' in dir(args):
        parser.print_help(sys.stderr)
        exit(1)

    if str(input_args['operation']).startswith('rsyslog-'):
        handle_rsyslog_operations(input_args)
        return


    # Execution should not come to this stage. If comes, then print help
    parser.print_help(sys.stderr)

if __name__ == '__main__':
    start()

exit(0)
