""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import argparse
import copy

####################################
# Constants for 'csadm log forward' starts here
###################################

LOG_COLLECT_SCRIPT = "/opt/cyops/scripts/log_collect.sh"
RSYSLOG_CONF_D = "/etc/rsyslog.d"
RSYSLOG_SERVER_DETAILS_FILE = RSYSLOG_CONF_D + "/server-details"
RSYSLOG_CONF_TOP_DIR = "/opt/cyops/configs/rsyslog"
RSYSLOG_SERVER_DETAILS_JSON = RSYSLOG_CONF_TOP_DIR + "/.server_details.json"
RSYSLOG_CERT_DIR = RSYSLOG_CONF_TOP_DIR + "/certs"
RSYSLOG_CONF_FILE_TARGET = RSYSLOG_CONF_D + "/00-rsyslog-fortisoar-settings.conf"
RSYSLOG_CONF_FILE_SRC = RSYSLOG_CONF_TOP_DIR + "/00-rsyslog-fortisoar-settings.conf"
APPLICATION_FILE_LIST_TO_WATCH = RSYSLOG_CONF_TOP_DIR + "/rsyslog-fortisoar-file-monitoring-list.conf"
RSYSLOG_LOG_FILE = "/var/log/cyops/csadm/log.log"
#
# We need following custom selinux policy package else we will get following error.
# SELinux is preventing /usr/sbin/rsyslogd from getattr access on the
# file /var/log/cyops/cyops-api/last_system_publish.log
#
CUSTOM_SELINUX_POLICY_PACKAGE = RSYSLOG_CONF_TOP_DIR + "/fortisoar-log-forwarding.pp"
#
# Installing selinux pp takes time, hence when first time it is installed, we will create this
# file and will not install when user tries to add another server.
#
CUSTOM_SELINUX_POLICY_PACKAGE_FLAG = RSYSLOG_CONF_TOP_DIR + "/.pp_installed"
PROTOCOL_UDP = "udp"
PROTOCOL_TCP = "tcp"
PROTOCOL_RELP = "relp"

TLS_CHOICES = ['true', 'false', '1', '0', 't', 'f']
TLS_YES_CHOICES = ['true', '1', 't']
DEFAULT_TLS = "false"
PROTOCOL_CHOICES = [PROTOCOL_TCP, PROTOCOL_UDP, PROTOCOL_RELP]

# NOTE: If you change the name of the export config file, make sure you change the file
# name at cyops-commonrpm/configs/opt/cyops/adm/subcmd/db in subroutine restore_config->syslog_config_json
EXPORT_CONFIG_JSON_FILE_NAME = "fsr-syslog-config.json"
EXPORT_FORMAT_JSON = 'json'
EXPORT_FORMAT_BASE64 = 'base64'
DEFAULT_EXPORT_FORMAT = EXPORT_FORMAT_JSON
EXPORT_FORMAT_CHOICES = [EXPORT_FORMAT_JSON, EXPORT_FORMAT_BASE64]

ADD_SERVER_HELP = "Add server which supports Rsyslog client"
SHOW_CONFIG_HELP = "Show configuration"
REMOVE_SERVER_HELP = "Remove server"
UPDATE_CONFIG_HELP = "Update the configuration"

LOG_FORWARDING_FEATURE_HELP = "Forward logs to the external log management server which supports Rsyslog client"

INPUT_ARGS_SERVER_IP_HELP = "Server name or IP address"
INPUT_ARGS_SERVER_PORT_HELP = "Server port"
INPUT_ARGS_SERVER_PROTOCOL_HELP = "Server protocol"
INPUT_ARGS_SERVER_TLS_HELP = "Enable TLS"
INPUT_ARGS_CA_CERT_HELP = "Path to the CA certificate PEM file which contains complete chain of CA certificates including file name. Required for TLS"
INPUT_ARGS_CLIENT_CERT_HELP = "Path to the client certificate PEM file including file name"
INPUT_ARGS_CLIENT_KEY_HELP = "Path to the client key PEM file including file name"

#
# In below argument's help section, the help starts with "R|".
# Note that it is NOT the error. It denotes the raw message.
# We wanted to add the newline(\n) in help section, hence we needed some way.
# The custom help fortmatter which we define
# looks for raw messages(starts with R|) and do the processing.
#

INPUT_ARGS_LOG_PATH_HELP = "R|Perform log collection.\nCollected logs are bundled into fortisoar-logs.tar.gz"
INPUT_ARGS_LOG_FILE_PASSWORD_HELP = "R|Password-protects the log file, i.e., users require the password to extract\nthe log file. Collected logs are bundled into fortisoar-logs.tar.gz.gpg"
INPUT_ARGS_JSON_FLAG_HELP = "Output in JSON format. Useful for automation"
INPUT_ARGS_CONFIG_NAME_HELP = "Configuration name"
INPUT_ARGS_UUID_HELP = "UUID of configuration"
INPUT_ARGS_EXPORT_PATH_HELP = "Directory path for exporting configuration"
INPUT_ARGS_EXPORT_FORMAT_HELP = "Export format. Default format is " + DEFAULT_EXPORT_FORMAT
INPUT_ARGS_PAYLOAD_HELP = "base64 encoded payload or path to JSON file"
INPUT_ARGS_NO_INTERACTION_HELP = "Don't ask for any confirmation"
INPUT_ARGS_CMD_SOURCE_HELP = argparse.SUPPRESS

INPUT_ARGS_LOG_FILE_PASSWORD = 'log_file_password'
INPUT_ARGS_LOG_PATH = 'log_path'
INPUT_ARGS_SERVER_IP = "server_name_or_ip"
INPUT_ARGS_SERVER_PORT = "server_port"
INPUT_ARGS_SERVER_PROTOCOL = "server_protocol"
INPUT_ARGS_SERVER_TLS = "server_tls"
INPUT_ARGS_CA_CERT = "ca_cert"
INPUT_ARGS_CLIENT_CERT = "client_cert"
INPUT_ARGS_CLIENT_KEY = "client_key"
INPUT_ARGS_FILTER = "filter"
INPUT_ARGS_JSON_FLAG = "output_in_json"
INPUT_ARGS_CONFIG_NAME = "config_name"
INPUT_ARGS_UUID = "uuid"
INPUT_ARGS_EXPORT_PATH = 'dir_path'
INPUT_ARGS_EXPORT_FORMAT = 'export_format'
INPUT_ARGS_PAYLOAD = 'import_payload'
INPUT_ARGS_NO_INTERACTION = "interactive"
INPUT_ARGS_CMD_SOURCE = 'cmd_source'


CA_CERT_PEM = "ca-cert.pem"
CLIENT_CERT_PEM = "client-cert.pem"
CLIENT_KEY_PEM = "client-key.pem"
REQUIRED_PEM_FILES_FOR_TLS = [CA_CERT_PEM, CLIENT_CERT_PEM, CLIENT_KEY_PEM]

#
# SDJ means server details JSON
#
SERVER_IP_IN_SDJ = 'server'
SERVER_PORT_IN_SDJ = 'port'
SERVER_PROTOCOL_IN_SDJ = 'protocol'
SERVER_TLS_IN_SDJ = 'tls'
CA_CERT_IN_SDJ = 'ca_cert'
CLIENT_CERT_IN_SDJ = 'client_cert'
CLIENT_KEY_IN_SDJ = 'client_key'
TLS_CERT_DIR_IN_SDJ = 'tls_cert_dir'
FILTER_IN_SDJ = "filter"
CONFIG_NAME_IN_SDJ = 'config_name'
UUID_IN_SDJ = 'uuid'
CMD_SOURCE_IN_SDJ = 'source'
SERVER_IPV6_IN_SDJ = "ipv6"

LIST_OF_COL_TO_HIDE_FROM_SDJ_DURING_SHOW_CONFIG = [CA_CERT_IN_SDJ, CLIENT_CERT_IN_SDJ, CLIENT_KEY_IN_SDJ, CMD_SOURCE_IN_SDJ, SERVER_IPV6_IN_SDJ]
FILTER_APPLICATION = "application"
FILTER_AUDIT = "audit"
FILTER_NONE = "none"
FILTER_ALL = [FILTER_APPLICATION, FILTER_AUDIT, FILTER_NONE]

DEFAULT_FILTER = copy.deepcopy(FILTER_ALL)

if FILTER_NONE in DEFAULT_FILTER:
    DEFAULT_FILTER.remove(FILTER_NONE)

DEFAULT_FILTER = ",".join(DEFAULT_FILTER)

INPUT_ARGS_FILTER_HELP =                          "Specify the comma separated filter for log forwarding, by default all logs will be forwarded.\n"
INPUT_ARGS_FILTER_HELP = INPUT_ARGS_FILTER_HELP + "For.eg if you want to forward only audit logs, then specify --filter=audit.\n"
INPUT_ARGS_FILTER_HELP = INPUT_ARGS_FILTER_HELP + "Valid filters are '" + ",".join(FILTER_ALL) + "'"

AUDIT_LOG_DEFINITION = "($syslogtag contains 'fortisoar-audit-logs')"

# OS logs are also considered under application logs
APPLICATION_LOG_DEFINITION = "not " + AUDIT_LOG_DEFINITION

### When more filter comes, then condition will be##########
#APPLICATION_LOG_DEFINITION = "((not " + AUDIT_LOG_DEFINITION + ") and (not " + PLAYBOOK_LOG_DEFINITION + "))"
###########

CMD_SOURCE_CLI = "cli"
CMD_SOURCE_UI = "ui"
CMD_SOURCE_ALL = [CMD_SOURCE_CLI, CMD_SOURCE_UI]
CMD_SOURCE_DEFAULT = CMD_SOURCE_CLI

CONFIG_START_COMMENT = "#### Config starts for uuid [{0}] here. Do not alter this line. ####"
CONFIG_END_COMMENT   = "#### Config ends for uuid [{0}] here. Do not alter this line.   ####"

LOG_FORWARD_VENDOR = "Fortinet Inc"
LOG_FORWARD_PRODUCT = "FortiSOAR"

FSR_FORWARD_TEMPLATE_NAME_NON_CEF = "fsr-forward-template-for-non-cef"
FSR_FORWARD_TEMPLATE_NAME_CEF =  "fsr-forward-template-for-cef"

FSR_FORWARD_FORMAT =                      "set $!fsr_version = \"{0}\";" + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + "set $!fsr_serial_number = \"{1}\";" + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + "set $!fsr_vd = \"{2}\";" + "\n\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + 'template(name="' + FSR_FORWARD_TEMPLATE_NAME_NON_CEF + '" type="string" ' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '         string="<%PRI%>%TIMESTAMP:::date-rfc3339% \\' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '                  %HOSTNAME% \\' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '                  %syslogtag:1:32%%msg:::sp-if-no-1st-sp%\\' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '                  CEF:0|' + LOG_FORWARD_VENDOR + '|\\' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '                  ' + LOG_FORWARD_PRODUCT + '|\\' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '                  %$!fsr_version%|\\' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '                  ' + "%programname%|%programname%|%syslogseverity%|\\" + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '                  devid=\\\"%$!fsr_serial_number%\\\" type=\\\"System Log\\\" vd=\\\"%$!fsr_vd%\\\" level=\\\"%syslogseverity-text%\\\" msg=\\\"%$!fsr_escaped_msg%\\\"\")' + "\n\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + 'template(name="' + FSR_FORWARD_TEMPLATE_NAME_CEF + '" type="string" ' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '         string="<%PRI%>%TIMESTAMP:::date-rfc3339% \\' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '                  %HOSTNAME% \\' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '                  %syslogtag:1:32%%msg:::sp-if-no-1st-sp%%msg%")' + "\n\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + 'if ( ($msg contains "CEF:0") ) then {{' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '    set $!fsr_forward_template = "' + FSR_FORWARD_TEMPLATE_NAME_CEF + '";' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '}} else {{' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '    set $!fsr_escaped_msg = replace($msg, "\\\"", "\\\\\\\"");' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '    set $!fsr_forward_template = "' + FSR_FORWARD_TEMPLATE_NAME_NON_CEF + '";' + "\n"
FSR_FORWARD_FORMAT = FSR_FORWARD_FORMAT + '}}' + "\n"

UDP_FORMAT_IN_SYSLOG =                        "if ( ($!fsr_forward_template contains 'non-cef') ) then {{" + "\n"
UDP_FORMAT_IN_SYSLOG = UDP_FORMAT_IN_SYSLOG + "    *.* @{0}:{1};" + FSR_FORWARD_TEMPLATE_NAME_NON_CEF      + "\n"
UDP_FORMAT_IN_SYSLOG = UDP_FORMAT_IN_SYSLOG + "}}else{{"                                                   + "\n"
UDP_FORMAT_IN_SYSLOG = UDP_FORMAT_IN_SYSLOG + "    *.* @{0}:{1};" + FSR_FORWARD_TEMPLATE_NAME_CEF          + "\n"
UDP_FORMAT_IN_SYSLOG = UDP_FORMAT_IN_SYSLOG + "}}"                                                         + "\n"

TCP_FORMAT_IN_SYSLOG =                        "if ( ($!fsr_forward_template contains 'non-cef') ) then {{" + "\n"
TCP_FORMAT_IN_SYSLOG = TCP_FORMAT_IN_SYSLOG + "    *.* @@{0}:{1};" + FSR_FORWARD_TEMPLATE_NAME_NON_CEF     + "\n"
TCP_FORMAT_IN_SYSLOG = TCP_FORMAT_IN_SYSLOG + "}}else{{"                                                   + "\n"
TCP_FORMAT_IN_SYSLOG = TCP_FORMAT_IN_SYSLOG + "    *.* @@{0}:{1};" + FSR_FORWARD_TEMPLATE_NAME_CEF         + "\n"
TCP_FORMAT_IN_SYSLOG = TCP_FORMAT_IN_SYSLOG + "}}"

RELP_FORMAT_IN_SYSLOG =                         "$ModLoad omrelp"     + "\n"
RELP_FORMAT_IN_SYSLOG = RELP_FORMAT_IN_SYSLOG + "if ( ($!fsr_forward_template contains 'non-cef') ) then {{"    + "\n"
RELP_FORMAT_IN_SYSLOG = RELP_FORMAT_IN_SYSLOG + "    *.* :omrelp:{0}:{1};" + FSR_FORWARD_TEMPLATE_NAME_NON_CEF  + "\n"
RELP_FORMAT_IN_SYSLOG = RELP_FORMAT_IN_SYSLOG + "}}else{{"                                                      + "\n"
RELP_FORMAT_IN_SYSLOG = RELP_FORMAT_IN_SYSLOG + "    *.* :omrelp:{0}:{1};" + FSR_FORWARD_TEMPLATE_NAME_CEF      + "\n"
RELP_FORMAT_IN_SYSLOG = RELP_FORMAT_IN_SYSLOG + "}}"                                                            + "\n"

TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT =                                                "$DefaultNetstreamDriver gtls"                               + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "$DefaultNetstreamDriverCAFile {2}"                          + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "$ActionSendStreamDriverAuthMode x509/certvalid"             + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "$ActionSendStreamDriverMode 1"                              + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "if ( ($!fsr_forward_template contains 'non-cef') ) then {{" + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "    *.* @@{0}:{1};" + FSR_FORWARD_TEMPLATE_NAME_NON_CEF     + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "}}else{{"                                                   + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "    *.* @@{0}:{1};" + FSR_FORWARD_TEMPLATE_NAME_CEF         + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "}}"                                                         + "\n"

TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT =                                             "$DefaultNetstreamDriver gtls"                               + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "$DefaultNetstreamDriverCAFile {2}"                          + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "$DefaultNetstreamDriverCertFile {3}"                        + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "$DefaultNetstreamDriverKeyFile {4}"                         + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "$ActionSendStreamDriverAuthMode x509/certvalid"             + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "$ActionSendStreamDriverMode 1"                              + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "if ( ($!fsr_forward_template contains 'non-cef') ) then {{" + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "    *.* @@{0}:{1};" + FSR_FORWARD_TEMPLATE_NAME_NON_CEF     + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "}}else{{"                                                   + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "    *.* @@{0}:{1};" + FSR_FORWARD_TEMPLATE_NAME_CEF     + "\n"
TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_TCP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "}}"

TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT =                                                 "$ModLoad omrelp"                                            + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "if ( ($!fsr_forward_template contains 'non-cef') ) then {{" + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "    action(type=\"omrelp\""                                 + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "           target=\"{0}\""                                  + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "           port=\"{1}\""                                    + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "           template=\"" + FSR_FORWARD_TEMPLATE_NAME_NON_CEF + "\"\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "           tls=\"on\")"                                     + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "}}else{{"                                                   + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "    action(type=\"omrelp\""                                 + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "           target=\"{0}\""                                  + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "           port=\"{1}\""                                    + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "           template=\"" + FSR_FORWARD_TEMPLATE_NAME_CEF     + "\"\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "           tls=\"on\")"                                     + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITHOUT_CLIENT_CERT + "}}"

TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT =                                              "$ModLoad omrelp"                                            + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "if ( ($!fsr_forward_template contains 'non-cef') ) then {{" + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "    action(type=\"omrelp\""                                 + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          target=\"{0}\""                                   + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          port=\"{1}\""                                     + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          template=\"" + FSR_FORWARD_TEMPLATE_NAME_NON_CEF  + "\"\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls=\"on\""                                       + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls.caCert=\"{2}\""                               + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls.myCert=\"{3}\""                               + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls.myPrivKey=\"{4}\""                            + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls.authMode=\"name\""                            + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls.permittedPeer=[{5}])"                         + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "}}else{{"                                                   + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "    action(type=\"omrelp\""                                 + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          target=\"{0}\""                                   + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          port=\"{1}\""                                     + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          template=\"" + FSR_FORWARD_TEMPLATE_NAME_CEF      + "\"\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls=\"on\""                                       + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls.caCert=\"{2}\""                               + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls.myCert=\"{3}\""                               + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls.myPrivKey=\"{4}\""                            + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls.authMode=\"name\""                            + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "          tls.permittedPeer=[{5}])"                         + "\n"
TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT = TLS_RELP_FORMAT_IN_SYSLOG_WITH_CLIENT_CERT + "}}"

####################################
# Constants for 'csadm log forward' ends here
###################################