""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import logging, sys

logger_formatter = logging.Formatter('%(asctime)s %(levelname)s %(module)s: %(funcName)s(): %(lineno)d PID %(process)d:\n%(message)s',
                                         datefmt='%Y-%m-%d,%H:%M:%S')

def _get_root_logger(name):
    root_logger = logging.getLogger(name)
    # We want every logger call like logger.info(), logger.warn() should get processes with
    # handlers, hence setting it to lower to DEBUG.
    # In normal case where we want to just write to file, it can be setup equal to the log level we want.
    # But here, we want logger which will display to console and writes to file.
    root_logger.setLevel(logging.DEBUG)
    return root_logger

def _get_file_handler(log_file_path, formatter=logger_formatter):
    file_handler = logging.FileHandler(log_file_path)
    file_handler.setFormatter(formatter)
    file_handler.setLevel(logging.NOTSET)
    return file_handler

def _get_console_handler():
    class MyFormatter(logging.Formatter):
        default_fmt = logger_formatter
        info_fmt = logging.Formatter('%(message)s')

        def format(self, record):
            if record.levelno == logging.INFO:
                return self.info_fmt.format(record)
            else:
                return self.default_fmt.format(record)

    console_handler = logging.StreamHandler(sys.stdout)
    myFormatter = MyFormatter()
    console_handler.setFormatter(myFormatter)
    return console_handler

def get_logger(name, log_file_path, level=logging.INFO):
    root_logger = _get_root_logger(name)
    console_handler = _get_console_handler()
    console_handler.setLevel(level)
    file_handler = _get_file_handler(log_file_path)
    root_logger.addHandler(console_handler)
    root_logger.addHandler(file_handler)
    #
    # Logger which can write to file as well as display to console is returned here.
    # Please note that the level here we are setting is for console printing. For file level,
    # it will be always that all messages will get logged to file.
    # So on console, if we want to see the DEBUG msgs as well, then set input parameter 'level' to DEBUG.
    #
    return root_logger