""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import json
from os.path import join
import re
import os
import sys
import requests, urllib3
from cshmac.requests import HmacAuth

CH_HOST = 'localhost'

#
# Please note that this script calls API's using localhost (variable declared 'CH_HOST').
# The environment which is executing this script may NOT have 'no_proxy' set with 'localhost'.
# In the environment in which 'https_proxy' is set and there is NO 'no_proxy' set for 'localhost',
# below API calls in this script will start failing. Hence, regardless of whether executing
# environment have 'no_proxy' set or not, define it here explicitly.
#
os.environ['no_proxy'] = "127.0.0.1, localhost"

PROXY_URL = 'https://' + CH_HOST + '/api/3/proxy'
PROXY_UUID = 'c90e8c1b-cu11-as33-3sk-717199a2e182'
PROXY_PATCH_URL = PROXY_URL + '/' + PROXY_UUID
KEY_PATH = '/opt/cyops/configs/keys/'
APPLIANCE_PUBLIC_KEY = ''
APPLIANCE_PRIVATE_KEY = ''

def read_files(key_path):
    global APPLIANCE_PUBLIC_KEY
    global APPLIANCE_PRIVATE_KEY
    with open(join(key_path, 'APPLIANCE_PRIVATE_KEY'), 'r') as private:
        APPLIANCE_PRIVATE_KEY = private.read().strip()
    with open(join(key_path, 'APPLIANCE_PUBLIC_KEY'), 'r') as public:
        APPLIANCE_PUBLIC_KEY = public.read().strip()

def get_appliance_keys():
    read_files(KEY_PATH)

def make_request(url, method, body=None):
    output = None
    try:
        urllib3.disable_warnings()
        get_appliance_keys()
        bodyless_methods = ['head', 'get']
        if method.lower() in bodyless_methods:
            body = None
        auth = HmacAuth(url, method, APPLIANCE_PUBLIC_KEY, APPLIANCE_PRIVATE_KEY, json.dumps(body))
        response = requests.request(method, url, auth=auth, json=body, verify=False)
        if response.status_code == 200 or response.status_code == 201 or response.status_code == 204:
            output = response.json()
        elif response.status_code == 503:
            output = response.json()
    except Exception as err:
        pass
    if output is None:
        msg =       "Unable to update the proxy in the database. Either the server is configured as\n"
        msg = msg + "passive HA node, or the API service is down.\n"
        msg = msg + "If this is a passive HA node, no further action is required. If not, you must restart\n"
        msg = msg + "all services and try again."
        print(msg)
    return output


def prepare_proxy_body(method, existing_proxy, url=None, port=None, username=None, password=None, iplist=[]):
    proxy_data = {}
    proxy_data['proxies'] = {}
    proxy_data['proxies']['HTTP_PROXY'] = {"url": "", "is_enabled": False, "port": None, "username": "", "password": ""}
    proxy_data['proxies']['HTTPS_PROXY'] = {"url": "", "is_enabled": False, "port": None, "username": "", "password": ""}
    proxy_data['toggle'] = False
    proxy_data['noProxy'] = ["127.0.0.1", "localhost"]

    if method == 'remove_proxy':
        already_configured_no_proxy_list = existing_proxy.get('noProxy', [])
        if len(already_configured_no_proxy_list) != 0:
            # preserve no_proxy list
            new_no_proxy_list = proxy_data['noProxy']
            new_no_proxy_list.extend(already_configured_no_proxy_list)
            proxy_data['noProxy'] = new_no_proxy_list
        return proxy_data

    elif method == 'http_proxy':
        http_proxy = {
            "url": url,
            "is_enabled": True,
            "port": int(port),
            "username": username,
            "password": password
        }
        if existing_proxy.get('proxies', {}).get('HTTP_PROXY'):
            existing_proxy['proxies']['HTTP_PROXY'] = http_proxy
            del(existing_proxy['proxies']['HTTPS_PROXY']['value'])
            return existing_proxy
        else:
            proxy_data['proxies']['HTTP_PROXY'] = http_proxy
            return proxy_data
    elif method == 'https_proxy':
        https_proxy = {
            "url": url,
            "is_enabled": True,
            "port": int(port),
            "username": username,
            "password": password
        }
        if existing_proxy.get('proxies', {}).get('HTTPS_PROXY'):
            existing_proxy['proxies']['HTTPS_PROXY'] = https_proxy
            del(existing_proxy['proxies']['HTTP_PROXY']['value'])
            return existing_proxy
        else:
            proxy_data['proxies']['HTTPS_PROXY'] = https_proxy
            return proxy_data
    elif method == 'no_proxy':
        if existing_proxy.get('proxies', {}):
            if existing_proxy.get('proxies', {}).get('HTTPS_PROXY'):
                del (existing_proxy['proxies']['HTTPS_PROXY']['value'])
                proxy_data['proxies']['HTTPS_PROXY'] = existing_proxy.get('proxies', {}).get('HTTPS_PROXY')
            if existing_proxy.get('proxies', {}).get('HTTP_PROXY'):
                del (existing_proxy['proxies']['HTTP_PROXY']['value'])
                proxy_data['proxies']['HTTP_PROXY'] = existing_proxy.get('proxies', {}).get('HTTP_PROXY')
        iplist.extend(proxy_data['noProxy'])
        proxy_data['noProxy'] = list(set(iplist))
        return proxy_data


def get_existing_proxy_method_url():
    existing_proxy = make_request(PROXY_URL, 'get')
    url = PROXY_URL
    method = 'post'
    if existing_proxy is None:
        return existing_proxy, url, method
    if existing_proxy.get('@id'):
        url = PROXY_PATCH_URL
        method = 'put'
    return existing_proxy, url, method

def delete_key_from_file(file, keys = []):
    regex = []
    for key in keys:
        regex.append(r'^\s*' + re.escape(key) + '\s*=')

    old_data = None
    new_data = []

    with open(file, "r") as fh:
        old_data = fh.readlines()

    for line in old_data:
        add_line = True
        for reg in regex:
            if re.search(reg, line):
                add_line = False
                break
        if add_line:
            new_data.append(line)

    with open(file, 'w') as out:
        out.writelines(new_data)

def is_proxy_configured_in_database():
    existing_proxy, URL, method = get_existing_proxy_method_url()
    if existing_proxy is None:
        return False, existing_proxy, URL
    http_proxy = existing_proxy.get('proxies', {}).get('HTTP_PROXY')
    https_proxy = existing_proxy.get('proxies', {}).get('HTTPS_PROXY')
    if http_proxy['url'] == "" and https_proxy['url'] == "":
        return False, existing_proxy, URL
    return True, existing_proxy, URL

def is_proxy_configured_in_system_files():
    with open("/etc/environment", "r") as fh:
        for line in fh.readlines():
             if re.search(r'^\s*' + re.escape("http_proxy") + '\s*=', line) or \
                     re.search(r'^\s*' + re.escape("https_proxy") + '\s*=', line):
                 return True
        return False

def remove_proxy_from_system_files():
    file = "/etc/environment"
    keys_to_delete = ['http_proxy', 'https_proxy']
    print("Removing proxy from file [" + file + "]")
    delete_key_from_file(file, keys_to_delete)
    file = "/etc/yum.conf"
    keys_to_delete = ['proxy', 'proxy_username', 'proxy_password']
    print("Removing proxy from file [" + file + "]")
    delete_key_from_file(file, keys_to_delete)

def remove_proxy_settings(update_env_only = False):
    if update_env_only:
        if not is_proxy_configured_in_system_files():
            print("Proxy is not configured")
            return False
        remove_proxy_from_system_files()
        return True

    is_configured_in_db, existing_proxy, URL = is_proxy_configured_in_database()
    if not is_configured_in_db:
        if not is_proxy_configured_in_system_files():
            print("Proxy is not configured")
            return False

    if existing_proxy is not None:
        proxy_body = prepare_proxy_body(existing_proxy=existing_proxy,
                                        method='remove_proxy')
        print("Removing proxy from database")
        make_request(URL, "PUT", proxy_body)
    remove_proxy_from_system_files()
    print("Successfully removed proxy")
    return True

def restore_proxy_settings():
    existing_proxy, URL, method = get_existing_proxy_method_url()
    # If proxy not set in database then existing_proxy return empty.
    if existing_proxy is None:
        return
    http_proxy = existing_proxy.get('proxies', {}).get('HTTP_PROXY')
    https_proxy = existing_proxy.get('proxies', {}).get('HTTPS_PROXY')
    no_proxy = existing_proxy.get('noProxy', [])
    if http_proxy and http_proxy['url'] != "" and http_proxy['is_enabled']:
        host = http_proxy['url']
        username = http_proxy['username']
        password = http_proxy['password']
        port = http_proxy['port']
        protocol = None
        if http_proxy['value'].startswith('https:'):
            protocol="https"
        elif http_proxy['value'].startswith('http:'):
            protocol="http"
        # The option --update-env-only update the /etc/yum.conf and /etc/environment files only.
        cmd = "sudo csadm network set-http-proxy --update-env-only --host " + host + " --protocol " + protocol
        if (username is not None) and username != "":
            cmd = cmd + " --user " + username
        if (password is not None) and password != "":
            cmd = cmd + " --password " + password
        if (port is not None) and str(port) != "":
            cmd = cmd + " --port " + str(port)
        os.system(cmd)

    if https_proxy and https_proxy['url'] != "" and https_proxy['is_enabled']:
        host = https_proxy['url']
        username = https_proxy['username']
        password = https_proxy['password']
        port = https_proxy['port']
        protocol = None
        if https_proxy['value'].startswith('https:'):
            protocol="https"
        elif https_proxy['value'].startswith('http:'):
            protocol="http"
        # The option --update-env-only update the /etc/yum.conf and /etc/environment files only.
        cmd = "sudo csadm network set-https-proxy --update-env-only --host " + host + " --protocol " + protocol
        if (username is not None) and username != "":
            cmd = cmd + " --user " + username
        if (password is not None) and password != "":
            cmd = cmd + " --password " + password
        if (port is not None) and str(port) != "":
            cmd = cmd + " --port " + str(port)
        os.system(cmd)

    if len(no_proxy) > 0:
        hosts = ','.join(no_proxy)
        # The option --update-env-only update the /etc/yum.conf and /etc/environment files only.
        cmd = "sudo csadm network set-no-proxy --update-env-only --host " + hosts
        os.system(cmd)

def list_proxy():
    file = "/etc/environment"
    regex = r'^\s*(no|http|https)_proxy\s*='
    data = None
    proxy_details = ""
    with open(file, "r") as fh:
        data = fh.readlines()
    for line in data:
        if re.search(regex, line):
            proxy_details = proxy_details + line
    if proxy_details == "":
       proxy_details = "No proxy configured"
    print (proxy_details.strip())

def setHttpProxy(url, port, username=None, password=None):
    existing_proxy, URL, method = get_existing_proxy_method_url()
    if existing_proxy is None:
        return False
    proxy_body = prepare_proxy_body(existing_proxy=existing_proxy, url=url, port=port, username=username,
                                    password=password, method='http_proxy')
    print("Setting http proxy in database")
    make_request(URL, method, proxy_body)
    return True


def setHttpsProxy(url, port, username=None, password=None):
    existing_proxy, URL, method = get_existing_proxy_method_url()
    if existing_proxy is None:
        return False
    proxy_body = prepare_proxy_body(existing_proxy=existing_proxy, url=url, port=port, username=username,
                                    password=password, method='https_proxy')
    print("Setting https proxy in database")
    make_request(URL, method, proxy_body)
    return True


def setNoProxy(comma_separated_iplist):
    iplist = comma_separated_iplist.split(',')
    existing_proxy, URL, method = get_existing_proxy_method_url()
    if existing_proxy is None:
        return False
    proxy_body = prepare_proxy_body(existing_proxy=existing_proxy, iplist=iplist, method='no_proxy')
    print("Setting no proxy list in database")
    make_request(URL, method, proxy_body)
    return True


def get_full_proxy_url(protocol, proxy_host, proxy_port, proxy_username=None, proxy_password=None):
    if proxy_host.startswith('http://') or proxy_host.startswith('https://'):
        proxy_host = proxy_host.split('//')[1]

    proxy_url = protocol + '://'
    if proxy_username is not None:
        proxy_url = proxy_url + proxy_username
        if proxy_password is not None:
             proxy_url = proxy_url + ':' + proxy_password
        proxy_url = proxy_url + '@'
    proxy_url = proxy_url + proxy_host + ':' + str(proxy_port)
    return proxy_url

def update_key_in_file(file, key, new_value, append_to_existing_value=False):
    regex = r'^\s*' + re.escape(key) + '\s*='
    existing_key_value = None
    existing_value = None
    data = None

    with open(file, "r") as fh:
        data = fh.readlines()

    for line in data:
        line = line.strip()
        if re.search(regex, line):
            existing_key_value = line

    if existing_key_value is not None:
        existing_value = existing_key_value.split('=')[1]
        # Remove the double quotes (") from start and end from the value
        existing_value = re.sub(r'^"|"$', '', existing_value) 
        # Remove the single quotes (') from start and end from the value
        existing_value = re.sub(r'^\'|\'$', '', existing_value)

    string_to_write = key + "=" + "'" + new_value + "'" + "\n"

    if append_to_existing_value: 
        if existing_value is not None:
            for value in existing_value.split(","):
                if value == new_value:
                    #Already added
                    return
            # Add new_value at the start of existing value
            new_value = new_value + "," + existing_value
            string_to_write = key + "=" + "'" + new_value + "'" + "\n"

    if existing_key_value is None:
        data.append(string_to_write)
    else:
        for index, line in reversed(list(enumerate(data))):
            if re.search(regex, line):
                data[index] = string_to_write
                break 

    with open(file, 'w') as out:
        out.writelines(data)

def update_etc_environment(key, value, append_to_existing_value=False):
    file = "/etc/environment"
    print("Updating " + file)
    update_key_in_file(file, key, value, append_to_existing_value)

def update_etc_yum_conf(proxy, proxy_username=None, proxy_password=None):
    file = "/etc/yum.conf"
    print("Updating " + file)
    proxy_regex = r'^\s*proxy\s*='
    proxy_username_regex = r'^\s*proxy_username\s*='
    proxy_password_regex = r'^\s*proxy_password\s*='
    updated_proxy = False
    updated_proxy_username = False
    updated_proxy_password = False
    data = None
    with open(file, "r") as fh:
        data = fh.readlines()
    new_data = []
    for line in data:
        if re.search(proxy_regex, line):
            new_data.append("proxy=" + proxy + "\n")
            updated_proxy = True
        elif re.search(proxy_username_regex, line):
            if proxy_username is not None:
                new_data.append("proxy_username=" + proxy_username + "\n") 
            updated_proxy_username = True
        elif re.search(proxy_password_regex, line):
            if proxy_password is not None:
                new_data.append("proxy_password=" + proxy_password + "\n")
            updated_proxy_password = True
        else:
            new_data.append(line)
    if not updated_proxy:
        new_data.append("proxy=" + proxy + "\n")
    if (not updated_proxy_username) and (proxy_username is not None):
        new_data.append("proxy_username=" + proxy_username + "\n")
    if (not updated_proxy_password) and (proxy_password is not None):
        new_data.append("proxy_password=" + proxy_password + "\n")
   
    with open(file, 'w') as out:
        out.writelines(new_data) 

if __name__ == "__main__":
    pass
