# Copyright start
# Copyright (C) 2008 - 2024 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

source_docker_config() {
    fsr_docker_config="/opt/cyops/scripts/docker/config"
    source $fsr_docker_config
}

create_docker_env_file() {
    local out
    out=$(xargs -0 -L1 -a /proc/1/environ)
    echo "Creating environment file $fdc_file_env"
    echo "$out" > "$fdc_file_env"
    chmod 644 "$fdc_file_env"
    local list_remove_shell_vars="PATH HOSTNAME TERM HOME"
    local shell_var
    for shell_var in $list_remove_shell_vars; do
        sed -i "/^${shell_var}=/d" "$fdc_file_env"
    done
}

#
# systemd resets the environment variables. The docker provisioning script is systemd service,
# and environment variables exported by 'docker run' are not by default available in 'env'.
# Reading them from linux environ file for pid 1.
#
get_env_variable() {
    local var_name=$1
    local out
    if [ -f "$fdc_file_env" ]; then
        out=$(cat "$fdc_file_env" | grep "^ *${var_name} *=" | cut -d'=' -f2)
    else
        out=$(xargs -0 -L1 -a /proc/1/environ | grep "^ *${var_name} *=" | cut -d'=' -f2)
    fi
    out=$(echo "$out" | xargs)
    out=$(echo "$out" | tr -d '"')
    out=$(echo "$out" | tr -d "'")
    echo "$out"
}

is_extension() {
    local check_type=$1

    if [ "X$check_type" = "Xstrict_check" ]; then
        if is_fmg || is_faz; then
            # 0 means true
            return 0
        fi
        # 1 means false
        return 1
    fi

    # Here comes, means user requested soft checking which is file based only.

    # 1 means false
    local flag_is_extn=1

    for flag_file in $LIST_FLAG_FILES; do
        if [ -f "$flag_file" ]; then
            # 0 means true
            flag_is_extn=0
            break
        fi
    done

    return $flag_is_extn
}

_is_ipv6_requested() {
    local is_ipv6=$(get_env_variable "$KEY_IPV6")
    # Convert all to lowercase
    is_ipv6=$(echo "$is_ipv6" | tr '[:upper:]' '[:lower:]')
    if [ "X$is_ipv6" = "Xtrue" ]; then
        # 0 means true
        return 0
    fi
    # 1 means false
    return 1
}

get_hostname_docker_host() {
    local hostname_load_balancer=$(get_env_variable "$KEY_HOSTNAME_LOAD_BALANCER")
    if [ "X$hostname_load_balancer" != "X" ]; then
        echo "$hostname_load_balancer"
        return
    fi
    local hostname_docker_host=$(get_env_variable "$KEY_HOSTNAME_DOCKER_HOST")
    if [ "X$hostname_docker_host" != "X" ]; then
        echo "$hostname_docker_host"
        return
    fi
    local hostname_extn
    hostname_extn=$($fdc_extn_settings --get-hostname)
    if [ $? -ne 0 ]; then
        return
    fi
    echo "$hostname_extn"
}

get_ip_docker_host() {
    local ip_docker_host=$(get_env_variable "$KEY_IP_DOCKER_HOST")
    if [ "X$ip_docker_host" != "X" ]; then
        echo "$ip_docker_host"
        return
    fi
    local ip_load_balancer=$(get_env_variable "$KEY_IP_LOAD_BALANCER")
    if [ "X$ip_load_balancer" != "X" ]; then
        echo "$ip_load_balancer"
        return
    fi
    local ip_extn
    ip_extn=$($fdc_extn_settings --get-ip)
    if [ $? -ne 0 ]; then
        return
    fi
    echo "$ip_extn"
}

# This indicates SME port on container, NOT on docker host
get_port_sme_container() {
    if is_extension; then
        echo "$fdc_extn_embd_sme_tcp_port"
        return
    fi
    echo "$fdc_embd_sme_tcp_port"
}

#
# This indicates UI port on docker host's.
#
get_port_ui_docker_host() {
    local port_ui=$(get_env_variable "$KEY_PORT_UI")
    if [ "X$port_ui" != "X" ]; then
        echo "$port_ui"
        return
    fi
    local https_port_extn
    https_port_extn=$($fdc_extn_settings --get-https-port)
    if [ $? -ne 0 ]; then
        return
    fi
    echo "$https_port_extn"
}

#
# This indicates SME port on docker host's.
#
get_port_sme_docker_host() {
    local port_sme=$(get_env_variable "$KEY_PORT_SME")
    if [ "X$port_sme" != "X" ]; then
        echo "$port_sme"
        return
    fi
    if is_extension; then
        echo "$fdc_extn_embd_sme_tcp_port"
        return
    fi
    echo "5671"
}

is_docker() {
    if [ ! -f "$fdc_flag_fsr_in_docker" ]; then
        # 1 means false
        return 1
    fi
    # 0 means true
    return 0
} 

is_fmg() {
    # 1 means false
    local flag_is_fmg=1

    if [ ! -f "$FILE_FSR_RUNNING_IN_FMG" ]; then
        return $flag_is_fmg
    fi

    local cmd="$fdc_extn_settings --check-if-fmg"
    if $cmd &>/dev/null; then
        # 0 means true
        flag_is_fmg=0
    fi
    return $flag_is_fmg
}

is_faz() {
    # 1 means false
    local flag_is_faz=1

    if [ ! -f "$FILE_FSR_RUNNING_IN_FAZ" ]; then
        return $flag_is_faz
    fi

    local cmd="$fdc_extn_settings --check-if-faz"
    if $cmd &>/dev/null; then
        # 0 means true
        flag_is_faz=0
    fi
    return $flag_is_faz
}

initialize_docker_global_variables() {
    export GLOBAL_VAR_HOSTNAME_DOCKER_HOST=$(get_hostname_docker_host)
    export GLOBAL_VAR_IP_DOCKER_HOST=$(get_ip_docker_host)
    export GLOBAL_VAR_PORT_UI_DOCKER_HOST=$(get_port_ui_docker_host)

    # Print global variables. This will be logged in log file, but very helpful for debugging,
    # since docker provisioning scripts uses them.
    echo "GLOBAL_VAR_HOSTNAME_DOCKER_HOST $GLOBAL_VAR_HOSTNAME_DOCKER_HOST"
    echo "GLOBAL_VAR_IP_DOCKER_HOST $GLOBAL_VAR_IP_DOCKER_HOST"
    echo "GLOBAL_VAR_PORT_UI_DOCKER_HOST $GLOBAL_VAR_PORT_UI_DOCKER_HOST"
}

notify_ui_fortisoar_state() {
    local payload=$1
    echo "Notifying UI with payload as $payload"
    $fdc_state_updater update-state --app-name "provisioning" \
                       --state-file $fdc_file_extn_fsr_provision \
                       --payload "$payload"

    chmod 644 $fdc_file_extn_fsr_provision
    chown nginx:nginx $fdc_file_extn_fsr_provision
}

add_docker_host_entry_in_etc_hosts() {
    if [ "X$GLOBAL_VAR_IP_DOCKER_HOST" = "X" ]; then
        # Currently, for enterprise docker, docker host IP is not exported. This is required only
        # for FMM/FAZ. For FMG/FAZ, /fortisoar is appended in all the calls from UI.
        # But it will only work if the UI is accessed through hostname of FMG/FAZ. Hence it is important
        # that FMG/FAZ hostname resolves from container.
        return
    fi
    local entry="$GLOBAL_VAR_IP_DOCKER_HOST $GLOBAL_VAR_HOSTNAME_DOCKER_HOST"
    local file_etc_hosts="/etc/hosts"
    echo "Adding entry in $file_etc_hosts [$entry]"
    echo "$entry" >> $file_etc_hosts
}

execute_container_backup() {
    csadm db --backup-config $fdc_dir_backup --minimal
}

enable_ipv6() {
    if ! _is_ipv6_requested; then
        return
    fi
    echo "Enabling IPV6"
    csadm network ipv6 --enable --skip-services-restart
}

update_offline_repo() {
    local hostname_repo=$(get_env_variable "$KEY_HOSTNAME_REPO")
    if [ "X$hostname_repo" = "X" ]; then
        return
    fi
    # Delete old entry
    echo "Updating repository hostname to $hostname_repo"
    sed -i "/product_yum_server=/d" /etc/environment
    echo "product_yum_server=$hostname_repo" >> /etc/environment
    echo "$hostname_repo" > /etc/yum/vars/product_yum_server
}

########
# Main/main/MAIN starts here
#####

#################
# How to use this?
# fsr_docker_utils_sh="/opt/cyops/scripts/docker/utils.sh"
# source $fsr_docker_utils_sh
##########

source_docker_config
