#!/usr/bin/env python3
""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import os, json, argparse, sys
import logging

# If you want to debug the script, then make below as logging.DEBUG
logging.basicConfig(level=logging.INFO)


def read_json(json_path):
    return json.load(open(json_path, "r"))


def is_empty(json_path):
    if os.stat(json_path).st_size == 0:
        return True
    else:
        return len(open(json_path).read().strip()) == 0

def write_json(json_path, contents):
    json_dump = json.dumps(contents, indent=4)
    fh = open(json_path, "w")
    fh.write(json_dump)
    fh.close()

"""
    {
        "provisioning": {
            "provision": true|false,
            "state": "init|restore|provisioned|failed|config_vm_failure_acknowledged",
            "ip": "192.168.51.173",
            "steps" : [
                {"label": "Config VM", "success": true/false},
                {"label": "Embedded SME Configuration", "success": true/false},
                {"label": "Extension License Installation", "success": true/false}
            ]
        }
   }
"""
def merge_payload(previous_payload, new_payload):
    for key_from_new in new_payload:
        if key_from_new in previous_payload:
            if isinstance(previous_payload[key_from_new], dict) and isinstance(new_payload[key_from_new], dict):
                merge_payload(previous_payload[key_from_new], new_payload[key_from_new])
            elif isinstance(previous_payload[key_from_new], list) and isinstance(new_payload[key_from_new], list):
                previous_payload[key_from_new].extend(new_payload[key_from_new])
            else:
                logging.debug("Updating key [" + key_from_new + "]")
                previous_payload[key_from_new] = new_payload[key_from_new]
        else:
            logging.debug("Adding key [" + key_from_new + "]")
            previous_payload[key_from_new] = new_payload[key_from_new]


def update_json_state_file(app_name, json_state_file_path, new_payload):
    previous_payload = {}
    if os.path.isfile(json_state_file_path) and (not is_empty(json_state_file_path)):
        previous_payload = read_json(json_state_file_path)
        if previous_payload and (app_name in previous_payload.keys()):
            previous_payload = previous_payload[app_name]

    logging.debug("Previous payload")
    logging.debug(previous_payload)
    logging.debug("New payload")
    logging.debug(new_payload)

    # merge new payload into previous
    merge_payload(previous_payload, new_payload)
    final_payload = {app_name: previous_payload}
    logging.debug("Final payload")
    logging.debug(final_payload)
    write_json(json_state_file_path, final_payload)


def clear_key(app_name,json_state_file_path,key_name):
    previous_payload = {}
    if os.path.isfile(json_state_file_path) and (not is_empty(json_state_file_path)):
        previous_payload = read_json(json_state_file_path)
        if previous_payload and (app_name in previous_payload.keys()):
            previous_payload = previous_payload[app_name]

    logging.debug("Previous payload")
    logging.debug(previous_payload)
    logging.debug("Key to be cleared")
    logging.debug(key_name)

    # clear key from previous payload
    previous_key = previous_payload.get(key_name)
    if previous_key:
        previous_key.clear()
    final_payload = previous_payload
    logging.debug("Final payload")
    logging.debug(final_payload)
    write_json(json_state_file_path, final_payload)

def define_parser(parser):
    subparsers = parser.add_subparsers()
    parser_update_state_file = subparsers.add_parser('update-state', help='Update fsr-state file')
    parser_update_state_file.set_defaults(operation='update-state')
    parser_clear_key = subparsers.add_parser('clear-key', help='Clear key in state file')
    parser_clear_key.set_defaults(operation='clear-key')

    parser_update_state_file.add_argument('--app-name',
                                          help="App name",
                                          dest='app_name',
                                          type=str,
                                          required=True)

    parser_update_state_file.add_argument('--state-file',
                                          help="Path of the JSON state file",
                                          dest='state_file_path',
                                          type=str,
                                          required=True)

    parser_update_state_file.add_argument('--payload',
                                          help='Payload in JSON format. Eg. {"key" : "value"}',
                                          dest='payload',
                                          type=str,
                                          required=True)

    parser_clear_key.add_argument('--key-name',
                                  help='Key to be cleared in app name',
                                  dest='key_name',
                                  type=str,
                                  required=True)

    parser_clear_key.add_argument('--app-name',
                                  help="App name",
                                  dest='app_name',
                                  type=str,
                                  required=True)

    parser_clear_key.add_argument('--state-file',
                                  help="Path of the JSON state file",
                                  dest='state_file_path',
                                  type=str,
                                  required=True)

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    define_parser(parser)
    args = parser.parse_args()
    if str(args.operation) == 'update-state':
        try:
            payload = json.loads(args.payload)
            update_json_state_file(args.app_name, args.state_file_path, payload)

        except Exception as e:
            print("Provide payload in json format")
            sys.exit(1)
    elif str(args.operation) == 'clear-key':
        clear_key(args.app_name,args.state_file_path,args.key_name)

    sys.exit(0)